

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
from balloon.common import print_e

import os


#--------------------------------------------------------------------------------------
def add_usage_description() :
    return " --rackspace-user=${RACKSPACE_USER} --rackspace-key=${RACKSPACE_KEY}"


#--------------------------------------------------------------------------------------
def add_parser_options( the_option_parser ) :
    the_option_parser.add_option( "--rackspace-user",
                                  metavar = "< Rackspace user >",
                                  action = "store",
                                  dest = "rackspace_user",
                                  help = "(${RACKSPACE_USER}, by default)",
                                  default = os.getenv( "RACKSPACE_USER" ) )
    
    the_option_parser.add_option( "--rackspace-key",
                                  metavar = "< Rackspace key >",
                                  action = "store",
                                  dest = "rackspace_key",
                                  help = "(${RACKSPACE_KEY}, by default)",
                                  default = os.getenv( "RACKSPACE_KEY" ) )
    pass


#--------------------------------------------------------------------------------------
def unpack( the_options ) :
    RACKSPACE_USER = the_options.rackspace_user
    RACKSPACE_KEY = the_options.rackspace_key

    return RACKSPACE_USER, RACKSPACE_KEY


#--------------------------------------------------------------------------------------
def compose_call( the_options ) :
    RACKSPACE_USER, RACKSPACE_KEY = unpack( the_options )

    a_call = "--rackspace-user=${RACKSPACE_USER} --rackspace-key=${RACKSPACE_KEY}"
    
    return a_call


#--------------------------------------------------------------------------------------
def extract_options( the_options ) :
    RACKSPACE_USER = the_options.rackspace_user
    if RACKSPACE_USER == None :
        print_e( "Define RACKSPACE_USER parameter through '--rackspace-user' option\n" )
        pass

    RACKSPACE_KEY = the_options.rackspace_key
    if RACKSPACE_KEY == None :
        print_e( "Define RACKSPACE_KEY parameter through '--rackspace-key' option\n" )
        pass

    return RACKSPACE_USER, RACKSPACE_KEY


#--------------------------------------------------------------------------------------



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script is responsible for the task packaging and sending it for execution in a cloud
"""


#--------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_e, sh_command, ssh_command, Timer

import balloon.rackspace as rackspace
import balloon.amazon as amazon

import sys, os, os.path, uuid


#--------------------------------------------------------------------------------------
# Defining utility command-line interface

an_usage_description = "%prog --task-control-dir=~/rackspace/control --task-control-dir=~/rackspace/data"
an_usage_description += common.add_usage_description()
an_usage_description += rackspace.add_usage_description()
an_usage_description += amazon.add_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
a_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

# Definition of the command line arguments
a_option_parser.add_option( "--task-control-dir",
                            metavar = "< location of the task 'control' scripts >",
                            action = "store",
                            dest = "task_control_dir",
                            help = "(\"%default\", by default)",
                            default = "./rackspace_control" )

a_option_parser.add_option( "--task-data-dir",
                            metavar = "< location of the task 'data' >",
                            action = "store",
                            dest = "task_data_dir",
                            help = "(\"%default\", by default)",
                            default = "./data" )

common.add_parser_options( a_option_parser )

rackspace.add_parser_options( a_option_parser )

amazon.add_parser_options( a_option_parser )
    
an_engine_dir = os.path.abspath( os.path.dirname( sys.argv[ 0 ] ) )


print_d( "\n---------------------------------------------------------------------------\n" )
# Extracting and verifying command-line arguments

an_options, an_args = a_option_parser.parse_args()

common.extract_options( an_options )

import os.path
a_task_control_dir = os.path.abspath( an_options.task_control_dir )
if not os.path.isdir( a_task_control_dir ) :
    print_e( "The task 'control' should a be directory\n" )
    pass

a_launch_script = os.path.join( a_task_control_dir, "launch" )
if not os.path.isfile( a_launch_script ) :
    print_e( "The task 'control' should contain 'launch' start-up script\n" )
    pass

a_task_data_dir = os.path.abspath( an_options.task_data_dir )
if not os.path.isdir( a_task_data_dir ) :
    print_e( "The task 'data' should a be directory\n" )
    pass

RACKSPACE_USER, RACKSPACE_KEY = rackspace.extract_options( an_options )

AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.extract_options( an_options )


print_d( "\n---------------------------------------------------------------------------\n" )
# Packaging of the local data

import os, tempfile
a_working_dir = tempfile.mkdtemp()
print_d( "a_working_dir = %s\n" % a_working_dir )

# Packaging the 'control' scripts
a_control_name = "task_control.tgz"
a_control_archive = os.path.join( a_working_dir, a_control_name )
sh_command( "cd %s && tar --exclude-vcs -czf %s *" % ( a_task_control_dir, a_control_archive ) )

# Packaging the task data
a_data_name = "task_data.tgz"
a_data_archive = os.path.join( a_working_dir, a_data_name )
sh_command( "cd %s && tar --exclude-vcs -czf %s *" % ( a_task_data_dir, a_data_archive ) )

# Packaging Python engine (itself)
sh_command( "cd %s && ./setup.py sdist" % an_engine_dir )
a_balloon_name = "balloon-0.5-alfa"
a_balloon_archive_name = a_balloon_name + os.extsep + "tar.gz"
a_balloon_source_archive = os.path.join( an_engine_dir, 'dist', a_balloon_archive_name )
a_balloon_target_archive = os.path.join( a_working_dir, a_balloon_archive_name )


print_d( "\n---------------------------------------------------------------------------\n" )
# Uploading task data into cloud

a_data_loading_time = Timer()

import cloudfiles
a_cloudfiles_conn = cloudfiles.get_connection( RACKSPACE_USER, RACKSPACE_KEY, timeout = 500 )
print_d( "a_cloudfiles_conn = %r\n" % a_cloudfiles_conn )

a_container_name = str( uuid.uuid4() )
a_cloudfiles_container = a_cloudfiles_conn.create_container( a_container_name )
print_d( "a_cloudfiles_container = %s\n" % a_cloudfiles_container.name )

a_file_object = a_cloudfiles_container.create_object( a_data_name )
a_file_object.load_from_filename( a_data_archive )
print_d( "a_file_object = %s\n" % a_file_object.name )

print_d( "a_data_loading_time = %s, sec\n" % a_data_loading_time )


print_d( "\n---------------------------------------------------------------------------\n" )
# Instanciating node in cloud
an_instance_reservation_time = Timer()

from libcloud.types import Provider 
from libcloud.providers import get_driver 

Driver = get_driver( Provider.RACKSPACE ) 
a_libcloud_conn = Driver( RACKSPACE_USER, RACKSPACE_KEY ) 
print_d( "a_libcloud_conn = %r\n" % a_libcloud_conn )

an_images = a_libcloud_conn.list_images() 
print_d( "an_images = %r\n" % an_images )

a_sizes = a_libcloud_conn.list_sizes() 
print_d( "a_sizes = %r\n" % a_sizes )

a_deployment_steps = []
from libcloud.deployment import MultiStepDeployment, ScriptDeployment, SSHKeyDeployment
a_key_pair_file = os.path.expanduser( "~/.ssh/id_rsa.pub")
a_deployment_steps.append( SSHKeyDeployment( open( a_key_pair_file ).read() ) )
# a_deployment_steps.append( ScriptDeployment( "apt-get -y install python-boto" ) )
# a_deployment_steps.append( ScriptDeployment( "apt-get -y install python-paramiko" ) )
# a_deployment_steps.append( ScriptDeployment( "apt-get -y install python-libcloud" ) )
# a_deployment_steps.append( ScriptDeployment( "apt-get -y install python-software-properties" ) )
# a_deployment_steps.append( ScriptDeployment( "add-apt-repository ppa:chmouel/rackspace-cloud-files" ) )
# a_deployment_steps.append( ScriptDeployment( "apt-get -y install python-rackspace-cloudfiles" ) )
a_msd = MultiStepDeployment( a_deployment_steps ) 

a_node_name = a_container_name
a_node = a_libcloud_conn.deploy_node( name = a_node_name, image = an_images[ 9 ] , size = a_sizes[ 0 ], deploy = a_msd ) 
print_d( "a_node = %r\n" % a_node )

print_d( "an_instance_reservation_time = %s, sec\n" % an_instance_reservation_time )


print_d( "\n---------------------------------------------------------------------------\n" )
# Uploading and running 'control' scripts into cloud
a_task_execution_time = Timer()

# Instantiating ssh connection with root access
import paramiko
a_ssh_client = paramiko.SSHClient()
a_ssh_client.set_missing_host_key_policy( paramiko.AutoAddPolicy() )
a_username = 'root'
a_ssh_client.connect( hostname = a_node.public_ip[ 0 ], port = 22, username = a_username )
print_d( 'ssh -i %s %s@%s\n' % ( a_key_pair_file, a_username, a_node.public_ip[ 0 ] ) )

# Preparing corresponding cloud 'working dir'
ssh_command( a_ssh_client, 'mkdir %s' % a_working_dir )

# Instantiating a sftp client
a_sftp_client = a_ssh_client.open_sftp()

# Uploading and installing into the cloud corresponding Python engine (itself)
a_sftp_client.put( a_balloon_source_archive, a_balloon_target_archive )
ssh_command( a_ssh_client, 'cd %s && tar -xzf %s' % ( a_working_dir, a_balloon_archive_name ) )
a_balloon_setup_dir = os.path.join( a_working_dir, a_balloon_name )
ssh_command( a_ssh_client, 'cd %s && python ./setup.py install' % ( a_balloon_setup_dir ) )

# Uploading and unpacking into the cloud 'control' scripts
a_sftp_client.put( a_control_archive, a_control_archive )
ssh_command( a_ssh_client, 'cd %s && tar -xzf %s' % ( a_working_dir, a_control_name ) )

# Executing into the cloud 'control' scripts
a_command = '%s/launch' % ( a_working_dir ) 
a_command += " --container-name='%s'" % a_container_name
a_command += " --data-name='%s'" % a_data_name
a_command += " --working-dir='%s'" % a_working_dir
a_command += " --rackspace-user='%s'" % RACKSPACE_USER
a_command += " --rackspace-key='%s'" % RACKSPACE_KEY
a_command += " --aws-access-key-id='%s'" % AWS_ACCESS_KEY_ID
a_command += " --aws-secret-access-key='%s'" % AWS_SECRET_ACCESS_KEY
ssh_command( a_ssh_client, a_command )

print_d( "a_task_execution_time = %s, sec\n" % a_task_execution_time )


print_d( "\n---------------------------------------------------------------------------\n" )
# Closing SSH connection
a_ssh_client.close()

# Cleaning tempral working folder
import shutil
shutil.rmtree( a_working_dir )


#---------------------------------------------------------------------------
# This refenrece value could be used further in cloud management pipeline
print a_container_name


print_d( "\n---------------------------------------------------------------------------\n" )
print_d( 'OK\n' )


#--------------------------------------------------------------------------------------



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script is responsible for the task packaging and sending it for execution in a cloud
"""


#--------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_list, print_e, sh_command, Timer

import balloon.rackspace as rackspace

import sys, os, os.path, uuid


#--------------------------------------------------------------------------------------
# Defining utility command-line interface

an_usage_description = "%prog --image-id=49 --size-id=1"
an_usage_description += common.add_usage_description()
an_usage_description += rackspace.add_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
an_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

an_option_parser.add_option( "--image-id",
                             metavar = "< Rackspace Servers Image ID >",
                             type = "int",
                             action = "store",
                             dest = "image_id",
                             help = "(%default, by default)",
                             default = "49" ) # Ubuntu 10.04 LTS (lucid)
an_option_parser.add_option( "--size-id",
                             metavar = "< Rackspace Servers Size ID >",
                             type = "int",
                             action = "store",
                             dest = "size_id",
                             help = "(%default, by default)",
                             default = "1" ) # RAM 256Mb HDD 10Gb

rackspace.add_parser_options( an_option_parser )
common.add_parser_options( an_option_parser )
    

#--------------------------------------------------------------------------------------
# Extracting and verifying command-line arguments

an_options, an_args = an_option_parser.parse_args()

an_enable_debug = common.extract_options( an_options )
RACKSPACE_USER, RACKSPACE_KEY = rackspace.extract_options( an_options )
an_image_id = an_options.image_id
a_size_id = an_options.size_id


print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
import sys
an_engine = sys.argv[ 0 ]

a_call = "%s --image-id=%d --size-id=%d %s" % ( an_engine, an_image_id, a_size_id, rackspace.compose_call( an_options ) )

print_d( a_call + '\n' )


print_d( "\n----------------------- Running actual functionality ----------------------\n" )
an_instance_reservation_time = Timer()

from libcloud.types import Provider 
from libcloud.providers import get_driver 

Driver = get_driver( Provider.RACKSPACE ) 
a_libcloud_conn = Driver( RACKSPACE_USER, RACKSPACE_KEY ) 
print_d( "a_libcloud_conn = %r\n" % a_libcloud_conn )


#--------------------------------------------------------------------------------------
an_images = a_libcloud_conn.list_images() 
print_list( "an_images :\n" , an_images )

an_image = None
for an_item in an_images :
    if int( an_item.id ) == an_image_id :
       an_image = an_item
       break
    pass

if an_image == None :
    an_option_parser.error( "--image-id='%d' does not exists" % an_image_id )
    pass

print_d( "an_image = %r\n" % an_image )


#--------------------------------------------------------------------------------------
a_sizes = a_libcloud_conn.list_sizes() 
print_list( "a_sizes :\n" , a_sizes )

a_size = None
for an_item in a_sizes :
    if int( an_item.id ) == a_size_id :
       a_size = an_item
       break
    pass

if a_size == None :
    an_option_parser.error( "--size-id='%d' does not exists" % a_size_id )
    pass

print_d( "a_size = %r\n" % a_size )


#--------------------------------------------------------------------------------------
a_node_name = str( uuid.uuid4() )
print_d( "a_node_name = '%s'\n" % a_node_name )

a_node = a_libcloud_conn.create_node( name = a_node_name, image = an_image , size = a_size ) 
print_d( "a_node = %r\n" % a_node )

print_d( "an_instance_reservation_time = %s, sec\n" % an_instance_reservation_time )


print_d( "\n------------------ Printing succussive pipeline arguments -----------------\n" )
a_password = a_node.extra.get( 'password' )
an_identity_file = "" # No identity file
a_host_port = 22
a_login_name = 'root'
a_host_name = a_node.public_ip[ 0 ]

print a_password
print an_identity_file
print a_host_port
print a_login_name
print a_host_name


print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
print_d( a_call + '\n' )
print_d( 'sshpass -p %s ssh -o "StrictHostKeyChecking no" -p %d %s@%s\n' % ( a_password, a_host_port, a_login_name, a_host_name ) )


print_d( "\n-------------------------------------- OK ---------------------------------\n" )



#!/usr/bin/env python

#------------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#------------------------------------------------------------------------------------------
"""
Deletes all SQS resources
"""

#------------------------------------------------------------------------------------------
import boto
import boto.ec2

import balloon.common as common
from balloon.common import print_d, print_i, print_e, sh_command, Timer, WorkerPool

import balloon.amazon as amazon


#------------------------------------------------------------------------------------------
# Defining utility command-line interface
an_usage_description = "%prog"
an_usage_description += common.add_usage_description()
an_usage_description += amazon.add_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
a_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

# Definition of the command line arguments
common.add_parser_options( a_option_parser )
amazon.add_parser_options( a_option_parser )


#------------------------------------------------------------------------------------------
# Extracting and verifying command-line arguments

an_options, an_args = a_option_parser.parse_args()

common.extract_options( an_options )

AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.extract_options( an_options )


print_d( "\n----------------------------- Delete SQS queues ---------------------------\n" )
a_sqs_conn = boto.connect_sqs( AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )
for a_queue in a_sqs_conn.get_all_queues() :
    print "'%s' : %d" % ( a_queue.name, a_queue.count() )
    a_queue.clear()
    a_queue.delete()
    pass

print


print_d( "\n-------------------------------------- OK ---------------------------------\n" )






#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
from balloon.common import print_e, print_d

import os, os.path, hashlib


#--------------------------------------------------------------------------------------
class TRootObject :
    "Represents S3 dedicated implementation of study root"

    def __init__( self, the_s3_conn, the_bucket, the_id ) :
        "Use static corresponding functions to an instance of this class"
        self._connection = the_s3_conn

        self._bucket = the_bucket
        self._id = the_id

        pass
    
    def __str__( self ) :

        return "'%s'- %s" % ( self._id, self._bucket )

    @staticmethod
    def get( AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY ) :
        "Looking for study root"
        import boto
        a_s3_conn = boto.connect_s3( AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )
        an_id = a_s3_conn.get_canonical_user_id()

        import hashlib
        a_bucket_name = hashlib.md5( an_id ).hexdigest()

        a_bucket = None
        try :
            a_bucket = a_s3_conn.get_bucket( a_bucket_name )
        except :
            a_bucket = a_s3_conn.create_bucket( a_bucket_name )
            pass
        
        return TRootObject( a_s3_conn, a_bucket, an_id )
    
    def _next( self ) :
        for a_study_key in self._bucket.list() :
            try :
                yield TStudyObject.get( self, get_key_name( a_study_key ) )
            except :
                print_d( "study '%s' has no corresponding bucket\n" % get_key_name( a_study_key ) )
                pass
            
            pass
        
        pass

    def __iter__( self ) :
        "Iterates through study files"
        
        return self._next()

    pass


#--------------------------------------------------------------------------------------
def api_version() :

    return '0.3'


#--------------------------------------------------------------------------------------
def _id_separator( the_api_version ) :
    if the_api_version == 'dummy' :
        return '|'

    return ' | '


#--------------------------------------------------------------------------------------
def generate_id( the_parent_id, the_child_name, the_api_version ) :
    a_separator = _id_separator( the_api_version )
    a_child_id = '%s%s%s' % ( the_parent_id, a_separator, the_child_name )

    a_bucket_name = hashlib.md5( a_child_id ).hexdigest()

    return a_child_id, a_bucket_name


#--------------------------------------------------------------------------------------
def _decorate_key_name( the_name ) :
    # This workaround make possible to use '/' symbol at the beginning of the key name

    return '# %s' % the_name


#--------------------------------------------------------------------------------------
def get_key_name( the_key ) :

    return the_key.name[ 2 : ]


#--------------------------------------------------------------------------------------
def get_key( the_parent_bucket, the_name ) :
    a_decorated_name = _decorate_key_name( the_name )

    from boto.s3.key import Key

    return Key( the_parent_bucket, a_decorated_name )


#--------------------------------------------------------------------------------------
class TStudyObject :
    "Represents S3 dedicated implementation of study object"

    def __init__( self, the_root_object, the_key, the_bucket, the_id, the_api_version ) :
        "Use static corresponding functions to an instance of this class"
        self._root_object = the_root_object

        self._key = the_key
        self._bucket = the_bucket
        self._id = the_id

        self._api_version = the_api_version

        pass
    
    def name( self ) :

        return get_key_name( self._key )

    def connection( self ) :

        return self._root_object._connection

    def __str__( self ) :

        return "'%s' - '%s' - %s" % ( self._id, self._api_version, self._bucket )

    @staticmethod
    def create( the_root_object, the_study_name ) :
        a_key = get_key( the_root_object._bucket, the_study_name )

        an_api_version = api_version()

        a_key.set_contents_from_string( an_api_version )

        an_id, a_bucket_name = generate_id( the_root_object._id, the_study_name, an_api_version )
    
        a_bucket = the_root_object._connection.create_bucket( a_bucket_name )

        return TStudyObject( the_root_object, a_key, a_bucket, an_id, an_api_version )

    @staticmethod
    def get( the_root_object, the_study_name ) :
        a_key = get_key( the_root_object._bucket, the_study_name )

        an_api_version = a_key.get_contents_as_string()

        an_id, a_bucket_name = generate_id( the_root_object._id, the_study_name, an_api_version )
    
        a_bucket = None
        try : # in case of broken entity
            a_bucket = the_root_object._connection.get_bucket( a_bucket_name )
        except :
            print_d( "study '%s' has no corresponding bucket\n" % the_study_name )
            pass
    
        return TStudyObject( the_root_object, a_key, a_bucket, an_id, an_api_version )

    def _next( self ) :
        if self._bucket == None :
            return 

        for a_file_key in self._bucket.list() :
            yield TFileObject.get( self, get_key_name( a_file_key ) )
            pass
        
        pass

    def __iter__( self ) :
        "Iterates through study files"
        
        return self._next()

    def delete( self ) :
        for a_file_object in self :
            a_file_object.delete()
            pass

        if self._bucket != None :
            self._bucket.delete()
            pass

        self._key.delete()
        pass

    pass


#--------------------------------------------------------------------------------------
def _file_key_separator( the_api_version ) :
    if the_api_version < '0.3' : 
        raise NotImplementedError( "The function _file_key_separator is not implemented in api vesion low 0.3" )

    return ' :: '


#--------------------------------------------------------------------------------------
def _extract_file_props( the_key, the_api_version ):
    a_contents = the_key.get_contents_as_string()
    a_separator = _file_key_separator( the_api_version )
    a_hex_md5, a_file_path = a_contents.split( a_separator )
    
    return a_hex_md5, a_file_path


#--------------------------------------------------------------------------------------
def generate_uploading_dir( the_file_path ) :
    a_file_dirname = os.path.dirname( the_file_path )
    a_file_basename = os.path.basename( the_file_path )

    a_sub_folder = hashlib.md5( a_file_basename ).hexdigest()
    a_working_dir = os.path.join( a_file_dirname, a_sub_folder )
    
    return a_working_dir


#--------------------------------------------------------------------------------------
class TFileObject :
    "Represents S3 dedicated implementation of file object"

    def __init__( self, the_study_object, the_key, the_bucket, the_id, the_hex_md5, the_file_path ) :
        "Use static corresponding functions to an instance of this class"
        self._study_object = the_study_object

        self._key = the_key
        self._bucket = the_bucket
        self._id = the_id

        self._hex_md5 = the_hex_md5
        self._file_path = the_file_path

        pass
    
    def file_path( self ) :
        
        return self._file_path

    def hex_md5( self ):
        
        return self._hex_md5

    def located_file( self ): 

        if self.api_version() < '0.3' or self.api_version() == 'dummy':

           return get_key_name( self._key )[ 1 : ]
        
        return get_key_name( self._key )
        
    def key( self ):
        
        return get_key_name( self._key )
    
    def connection( self ) :

        return self._study_object._connection

    def api_version( self ) :

        return self._study_object._api_version

    def __str__( self ) :

        return "'%s' - '%s' - %s" % ( self._id, self._hex_md5, self._bucket )

    @staticmethod
    def create( the_study_object, the_file_path, the_file_location, the_hex_md5 ) :
        a_file_name = os.path.basename( the_file_path )
        
        a_located_file = os.path.join( the_file_location, a_file_name )

        a_key = get_key( the_study_object._bucket, a_located_file )
        
        an_api_version = the_study_object._api_version

        a_separator = _file_key_separator( an_api_version )
        
        a_key.set_contents_from_string( the_hex_md5 + a_separator + the_file_path )
        
        an_id, a_bucket_name = generate_id( the_study_object._id, a_located_file, an_api_version )
    
        a_bucket = the_study_object.connection().create_bucket( a_bucket_name )

        return TFileObject( the_study_object, a_key, a_bucket, an_id, the_hex_md5, the_file_path )

    @staticmethod
    def get( the_study_object, the_file_key ) :
        a_key = get_key( the_study_object._bucket, the_file_key )
        
        an_api_version = the_study_object._api_version

        a_hex_md5, a_file_path = None, None
        try : # in case of broken entity
            if the_study_object._api_version >= "0.3" and the_study_object._api_version != 'dummy':
                a_hex_md5, a_file_path = _extract_file_props( a_key, an_api_version )
            else:
                a_file_path = get_key_name( a_key )
                a_hex_md5 = a_key.get_contents_as_string()
                pass
        except :
            pass
    
        an_id, a_bucket_name = generate_id( the_study_object._id, the_file_key, an_api_version )

        a_bucket = the_study_object.connection().get_bucket( a_bucket_name )

        return TFileObject( the_study_object, a_key, a_bucket, an_id, a_hex_md5, a_file_path )

    def _next( self ) :
        for a_seed_key in self._bucket.list() :
            yield TSeedObject.get( self, a_seed_key )
            
            pass
        
        pass

    def __iter__( self ) :
        "Iterates through file items"
        
        return self._next()

    def delete( self ) :
        for a_seed_object in self :
            a_seed_object.delete()
        
            pass

        self._bucket.delete()
        self._key.delete()

        pass

    def seal_name( self ) :

        return self._bucket.name

    def seal( self, the_working_dir ) :
        "To mark the everything was sucessfuly uploaded"
        os.rmdir( the_working_dir )

        # To mark that final file item have been sucessfully uploaded
        a_seed_key = get_key( self._bucket, self.seal_name() )
        a_seed_key.set_contents_from_string( 'seal' )

        pass

    pass


#--------------------------------------------------------------------------------------
def _seed_key_separator( the_api_version ) :
    if the_api_version == 'dummy' :
        return ':'

    return ' % '


#--------------------------------------------------------------------------------------
def generate_seed_name( the_hex_md5, the_file_seed, the_api_version ) :
    a_separator = _seed_key_separator( the_api_version )

    if the_api_version == 'dummy' :
        return '%s%s%s' % ( the_file_seed, a_separator, the_hex_md5 )

    if the_api_version == '0.1' :
        return '%s%s%s' % ( the_hex_md5, a_separator, the_file_seed )

    return '%s%s%s' % ( the_file_seed, a_separator, the_hex_md5 )


#--------------------------------------------------------------------------------------
def extract_seed_props( the_seed_key_name, the_api_version ) :
    a_separator = _seed_key_separator( the_api_version )

    a_seed_name, a_hex_md5 = None, None
    try :
        if the_api_version == 'dummy' :
            a_seed_name, a_hex_md5 = the_seed_key_name.split( a_separator )
        elif the_api_version == '0.1' :
            a_hex_md5, a_seed_name = the_seed_key_name.split( a_separator )
        else :
            a_seed_name, a_hex_md5 = the_seed_key_name.split( a_separator )
            pass
    except :
        pass

    return a_hex_md5, a_seed_name


#--------------------------------------------------------------------------------------
class TSeedObject :
    "Represents S3 dedicated implementation of item object"

    def __init__( self, the_file_object, the_key, the_name, the_hex_md5 ) :
        "Use static corresponding functions to an instance of this class"
        self._file_object = the_file_object

        self._key = the_key
        self._name = the_name
        self._hex_md5 = the_hex_md5

        pass
    
    def name( self ) :

        return self._name

    def hex_md5( self ) :

        return self._hex_md5

    def is_seal( self ) :
        
        return self._name == None

    def download( self, the_file_path ) :
        self._key.get_contents_to_filename( the_file_path )

        pass

    def __str__( self ) :

        return "%s" % ( self._key )

    @staticmethod
    def create( the_file_object, the_seed_name, the_seed_path ) :
        a_file_pointer = open( the_seed_path, 'rb' )

        from balloon.common import compute_md5
        a_md5 = compute_md5( a_file_pointer )
        a_hex_md5, a_base64md5 = a_md5

        an_api_version = the_file_object.api_version()
        a_seed_name = generate_seed_name( a_hex_md5, the_seed_name, an_api_version )

        a_seed_key = get_key( the_file_object._bucket, a_seed_name )
        # a_part_key.set_contents_from_file( a_file_pointer, md5 = a_md5 ) # this method is not thread safe
        a_seed_key.set_contents_from_file( a_file_pointer, headers = { 'Content-Type' : 'application/x-tar' } )
        
        a_file_pointer.close()
        os.remove( the_seed_path )

        return TSeedObject( the_file_object, a_seed_key, the_seed_name, a_hex_md5 )

    @staticmethod
    def get( the_file_object, the_seed_key ) :
        an_api_version = the_file_object.api_version()

        a_hex_md5, a_seed_name = extract_seed_props( get_key_name( the_seed_key ), an_api_version )

        return TSeedObject( the_file_object, the_seed_key, a_seed_name, a_hex_md5 )

    def delete( self ) :
        self._key.delete()

        pass

    pass


#--------------------------------------------------------------------------------------



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script is responsible for the task packaging and sending it for execution in a cloud
"""


#--------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_e, sh_command, ssh_command, Timer

import balloon.amazon as amazon
from balloon.amazon import wait_activation, wait_ssh

import sys, os, os.path, uuid


#--------------------------------------------------------------------------------------
# Defining utility command-line interface

an_usage_description = "%prog --task-control-dir=~/rackspace/control --task-control-dir=~/rackspace/data"
an_usage_description += common.add_usage_description()
an_usage_description += amazon.add_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
a_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

# Definition of the command line arguments
a_option_parser.add_option( "--task-control-dir",
                            metavar = "< location of the task 'control' scripts >",
                            action = "store",
                            dest = "task_control_dir",
                            help = "(\"%default\", by default)",
                            default = "./amazon_control" )

a_option_parser.add_option( "--task-data-dir",
                            metavar = "< location of the task 'data' >",
                            action = "store",
                            dest = "task_data_dir",
                            help = "(\"%default\", by default)",
                            default = "./data" )

common.add_parser_options( a_option_parser )

amazon.add_parser_options( a_option_parser )
    
an_engine_dir = os.path.abspath( os.path.dirname( sys.argv[ 0 ] ) )


print_d( "\n---------------------------------------------------------------------------\n" )
# Extracting and verifying command-line arguments

an_options, an_args = a_option_parser.parse_args()

common.extract_options( an_options )

import os.path
a_task_control_dir = os.path.abspath( an_options.task_control_dir )
if not os.path.isdir( a_task_control_dir ) :
    print_e( "The task 'control' should a be directory\n" )
    pass

a_launch_script = os.path.join( a_task_control_dir, "launch" )
if not os.path.isfile( a_launch_script ) :
    print_e( "The task 'control' should contain 'launch' start-up script\n" )
    pass

a_task_data_dir = os.path.abspath( an_options.task_data_dir )
if not os.path.isdir( a_task_data_dir ) :
    print_e( "The task 'data' should a be directory\n" )
    pass

AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.extract_options( an_options )


print_d( "\n---------------------------------------------------------------------------\n" )
# Packaging of the local data

import os, tempfile
a_working_dir = tempfile.mkdtemp()
print_d( "a_working_dir = %s\n" % a_working_dir )

# Packaging the 'control' scripts
a_control_name = "task_control.tgz"
a_control_archive = os.path.join( a_working_dir, a_control_name )
sh_command( "cd %s && tar --exclude-vcs -czf %s *" % ( a_task_control_dir, a_control_archive ) )

# Packaging the task data
a_data_name = "task_data.tgz"
a_data_archive = os.path.join( a_working_dir, a_data_name )
sh_command( "cd %s && tar --exclude-vcs -czf %s *" % ( a_task_data_dir, a_data_archive ) )

# Packaging Python engine (itself)
sh_command( "cd %s && ./setup.py sdist" % an_engine_dir )
a_balloon_name = "balloon-0.5-alfa"
a_balloon_archive_name = a_balloon_name + os.extsep + "tar.gz"
a_balloon_source_archive = os.path.join( an_engine_dir, 'dist', a_balloon_archive_name )
a_balloon_target_archive = os.path.join( a_working_dir, a_balloon_archive_name )


print_d( "\n---------------------------------------------------------------------------\n" )
# Uploading task data into cloud
a_data_loading_time = Timer()

import boto
a_s3_conn = boto.connect_s3( AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )
print_d( "a_s3_conn = %r\n" % a_s3_conn )
print_d( "a_s3_conn.get_canonical_user_id() = %s\n" % a_s3_conn.get_canonical_user_id() )

a_bucket_name = str( uuid.uuid4() )
a_s3_bucket = a_s3_conn.create_bucket( a_bucket_name )
print_d( "a_s3_bucket = %s\n" % a_s3_bucket.name )

from boto.s3.key import Key
a_s3_bucket_key = Key( a_s3_bucket )
a_s3_bucket_key.key = a_data_name
a_s3_bucket_key.set_contents_from_filename( a_data_archive )
print_d( "a_s3_bucket_key = %s\n" % a_s3_bucket_key.name )

print_d( "a_data_loading_time = %s, sec\n" % a_data_loading_time )


print_d( "\n---------------------------------------------------------------------------\n" )
# Instanciating node in cloud
an_instance_reservation_time = Timer()

import boto
# Establish an connection with EC2
an_ec2_conn = boto.connect_ec2()
print_d( '%r\n' % an_ec2_conn )

# Choose an image to be run
an_image_ids = "ami-2d4aa444"
# an_image_ids = "ami-fd4aa494" # 64-bit version
an_images = an_ec2_conn.get_all_images( image_ids = an_image_ids )
an_image = an_images[ 0 ]
print_d( '%s\n' % an_image.location )

# Generating an unique name to be used for corresponding
# ssh "key pair" & EC2 "security group"
import tempfile
an_unique_file = tempfile.mkstemp()[ 1 ]
an_unique_name = os.path.basename( an_unique_file )
os.remove( an_unique_file )
print_d( 'an_unique_name = %s\n' % an_unique_name )

# Asking EC2 to generate a new ssh "key pair"
a_key_pair_name = an_unique_name
a_key_pair = an_ec2_conn.create_key_pair( a_key_pair_name )

# Saving the generated ssh "key pair" locally
a_key_pair_dir = os.path.expanduser( "~/.ssh")
a_key_pair.save( a_key_pair_dir )
a_key_pair_file = os.path.join( a_key_pair_dir, a_key_pair.name ) + os.path.extsep + "pem"
print_d( 'a_key_pair_file = %s\n' % a_key_pair_file )

import stat
os.chmod( a_key_pair_file, stat.S_IRUSR )

# Asking EC2 to generate a new "sequirity group" & apply corresponding firewall permissions
a_security_group = an_ec2_conn.create_security_group( an_unique_name, 'temporaly generated' )
a_security_group.authorize( 'tcp', 80, 80, '0.0.0.0/0' )
a_security_group.authorize( 'tcp', 22, 22, '0.0.0.0/0' )

# Creating a EC2 "reservation" with all the parameters mentioned above
an_instance_type = 'm1.small'
# an_instance_type = 'm1.large' # 64-bit version
a_reservation = an_image.run( instance_type = an_instance_type, min_count = 1, max_count = 1, key_name = a_key_pair_name, security_groups = [ a_security_group.name ] )
an_instance = a_reservation.instances[ 0 ]
wait_activation( an_instance )

# Instantiating ssh connection with root access
import paramiko
a_ssh_client = paramiko.SSHClient()
a_ssh_client.set_missing_host_key_policy( paramiko.AutoAddPolicy() )
a_rsa_key = paramiko.RSAKey( filename = a_key_pair_file )

a_username = 'ubuntu'
a_ssh_connect = lambda : a_ssh_client.connect( hostname = an_instance.dns_name, port = 22, username = a_username, pkey = a_rsa_key )

# Making sure that corresponding instances are ready to use
wait_ssh( a_ssh_connect, a_ssh_client, 'echo  > /dev/null' )
print_d( 'ssh -i %s %s@%s\n' % ( a_key_pair_file, a_username, an_instance.dns_name ) )

print_d( "an_instance_reservation_time = %s, sec\n" % an_instance_reservation_time )


print_d( "\n---------------------------------------------------------------------------\n" )
# Uploading and running 'control' scripts into cloud
a_task_execution_time = Timer()

# Preparing corresponding cloud 'working dir'
ssh_command( a_ssh_client, 'mkdir %s' % a_working_dir )

# Instantiating a sftp client
a_sftp_client = a_ssh_client.open_sftp()

# Uploading and installing into the cloud corresponding Python engine (itself)
a_sftp_client.put( a_balloon_source_archive, a_balloon_target_archive )
ssh_command( a_ssh_client, 'cd %s && tar -xzf %s' % ( a_working_dir, a_balloon_archive_name ) )
a_balloon_setup_dir = os.path.join( a_working_dir, a_balloon_name )
ssh_command( a_ssh_client, 'cd %s && sudo python ./setup.py install' % ( a_balloon_setup_dir ) )

# Uploading and unpacking into the cloud 'control' scripts
a_sftp_client.put( a_control_archive, a_control_archive )
ssh_command( a_ssh_client, 'cd %s && tar -xzf %s' % ( a_working_dir, a_control_name ) )

# Executing into the cloud 'control' scripts
a_command = '%s/launch' % ( a_working_dir ) 
a_command += " --bucket-name='%s'" % a_bucket_name
a_command += " --data-name='%s'" % a_data_name
a_command += " --working-dir='%s'" % a_working_dir
a_command += " --aws-access-key-id='%s'" % AWS_ACCESS_KEY_ID
a_command += " --aws-secret-access-key='%s'" % AWS_SECRET_ACCESS_KEY
# ssh_command( a_ssh_client, a_command )

# ssh_command( a_ssh_client, 'sudo add-apt-repository ppa:cae-team/ppa' )
# ssh_command( a_ssh_client, 'sudo apt-get update' )
# ssh_command( a_ssh_client, 'sudo apt-get upgrade' )
# ssh_command( a_ssh_client, 'sudo apt-get install openfoam-dev-1.5' )

print_d( "a_task_execution_time = %s, sec\n" % a_task_execution_time )


print_d( "\n---------------------------------------------------------------------------\n" )
# Closing SSH connection
a_ssh_client.close()

# Cleaning tempral working folder
import shutil
shutil.rmtree( a_working_dir )


#---------------------------------------------------------------------------
# This refenrece value could be used further in cloud management pipeline
print a_bucket_name


print_d( "\n---------------------------------------------------------------------------\n" )
print_d( 'OK\n' )


#--------------------------------------------------------------------------------------





#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
from balloon.common import print_e, print_d, Timer

import os, os.path


#--------------------------------------------------------------------------------------
def add_usage_description() :
    return " --image-location='us-east-1' --reservation-id='r-8cc1dfe7' --identity-file='~/.ssh/tmpaSRNcY.pem' --host-port=22 --login-name='ubuntu'"


#--------------------------------------------------------------------------------------
def add_parser_options( the_option_parser ) :
    the_option_parser.add_option( "--image-location",
                                  metavar = "< location of the AMI >",
                                  action = "store",
                                  dest = "image_location",
                                  help = "(\"%default\", by default)",
                                  default = None )
    the_option_parser.add_option( "--reservation-id",
                                  metavar = "< Amazon EC2 Reservation ID >",
                                  action = "store",
                                  dest = "reservation_id",
                                  help = "(\"%default\", by default)",
                                  default = None )
    the_option_parser.add_option( "--identity-file",
                                  metavar = "< selects a file from which the identity (private key) for RSA or DSA authentication is read >",
                                  action = "store",
                                  dest = "identity_file",
                                  default = None )
    the_option_parser.add_option( "--host-port",
                                  metavar = "< port to be used >",
                                  type = "int",
                                  action = "store",
                                  dest = "host_port",
                                  default = 22 )
    the_option_parser.add_option( "--login-name",
                                  metavar = "< specifies the user to log in as on the remote machine >",
                                  action = "store",
                                  dest = "login_name",
                                  help = "(\"%default\", by default)",
                                  default = 'ubuntu' )
    pass


#--------------------------------------------------------------------------------------
def unpack( the_options ) :
    an_image_location = the_options.image_location
    a_reservation_id = the_options.reservation_id
    an_identity_file = the_options.identity_file
    a_host_port = the_options.host_port
    a_login_name = the_options.login_name

    return an_image_location, a_reservation_id, an_identity_file, a_host_port, a_login_name


#--------------------------------------------------------------------------------------
def compose_call( the_options ) :
    an_image_location, a_reservation_id, an_identity_file, a_host_port, a_login_name = unpack( the_options )

    a_call = "--image-location='%s' --reservation-id='%s' --identity-file='%s' --host-port=%d --login-name='%s'" % \
        ( an_image_location, a_reservation_id, an_identity_file, a_host_port, a_login_name )
    
    return a_call


#--------------------------------------------------------------------------------------
def extract_options( the_options ) :
    an_image_location = the_options.image_location
    if an_image_location == None :
        an_image_location = raw_input()
        pass
    the_options.image_location = an_image_location

    a_reservation_id = the_options.reservation_id
    if a_reservation_id == None :
        a_reservation_id = raw_input()
        pass
    the_options.reservation_id = a_reservation_id
    
    an_identity_file = the_options.identity_file
    if an_identity_file == None :
        an_identity_file = raw_input()
        pass
    import os.path
    an_identity_file = os.path.expanduser( an_identity_file )
    an_identity_file = os.path.abspath( an_identity_file )
    the_options.identity_file = an_identity_file
    
    a_host_port = the_options.host_port
    if a_host_port == None :
        a_host_port = int( raw_input() )
        pass
    the_options.host_port = a_host_port

    a_login_name = the_options.login_name
    
    return an_image_location, a_reservation_id, an_identity_file, a_host_port, a_login_name


#--------------------------------------------------------------------------------------
def print_options( the_image_location, the_reservation_id, the_identity_file, the_host_port, the_login_name = 'ubuntu' ) :

    print the_image_location
    print the_reservation_id
    print the_identity_file
    print the_host_port
    print the_login_name

    pass


#--------------------------------------------------------------------------------------
def get_reservation( the_ec2_conn, the_reservation_id ) :
    for a_reservation in the_ec2_conn.get_all_instances() :
        if a_reservation.id == the_reservation_id :
            an_instance = a_reservation.instances[ 0 ]
            a_status = an_instance.update()
            if a_status != 'terminated' :
                print_d( '< %r > : %s\n' % ( a_reservation, a_reservation.instances ) )
                return a_reservation
            pass
        pass
    pass


#--------------------------------------------------------------------------------------
def get_security_group( the_ec2_conn, the_reservation ) :
    a_security_group = the_ec2_conn.get_all_security_groups( [ the_reservation.groups[ 0 ].id ] )[ 0 ]
    print_d( "< %r > : %s\n" % ( a_security_group, a_security_group.rules ) )

    return a_security_group


#--------------------------------------------------------------------------------------




#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
from balloon.common import print_e, print_d, Timer


#--------------------------------------------------------------------------------------
def add_usage_description() :
    return " [ --password='P@ssw0rd' | --identity-file='~/.ssh/tmpraDD2j.pem' ] --host-port=22 --login-name='ubuntu' --host-name='ec2-184-73-11-20.compute-1.amazonaws.com'"


#--------------------------------------------------------------------------------------
def add_parser_options( the_option_parser ) :
    the_option_parser.add_option( "--password",
                                  metavar = "< password for the given host and login name >",
                                  action = "store",
                                  dest = "password",
                                  default = "" )
    the_option_parser.add_option( "--identity-file",
                                  metavar = "< selects a file from which the identity (private key) for RSA or DSA authentication is read >",
                                  action = "store",
                                  dest = "identity_file",
                                  default = "" )
    the_option_parser.add_option( "--host-port",
                                  metavar = "< port to be used >",
                                  type = "int",
                                  action = "store",
                                  dest = "host_port",
                                  default = None )
    the_option_parser.add_option( "--login-name",
                                  metavar = "< specifies the user to log in as on the remote machine >",
                                  action = "store",
                                  dest = "login_name",
                                  help = "(\"%default\", by default)",
                                  default = None )
    the_option_parser.add_option( "--host-name",
                                  metavar = "< instance public DNS >",
                                  action = "store",
                                  dest = "host_name",
                                  default = None )
    the_option_parser.add_option( "--command",
                                  metavar = "< command to be run on the remote machine >",
                                  action = "store",
                                  dest = "command",
                                  help = "('%default', by default)",
                                  default = "echo  > /dev/null" )
    return the_option_parser


#--------------------------------------------------------------------------------------
def unpack( the_options ) :
    a_password = the_options.password
    an_identity_file = the_options.identity_file
    a_host_port = the_options.host_port
    a_login_name = the_options.login_name
    a_host_name = the_options.host_name
    a_command = the_options.command

    return a_password, an_identity_file, a_host_port, a_login_name, a_host_name, a_command


#--------------------------------------------------------------------------------------
def print_call( the_password, the_identity_file, the_host_port, the_login_name, the_host_name, the_command = None ) :
    if the_password != "" :
        print_d( 'sshpass -p %s ssh -o "StrictHostKeyChecking no" -p %d %s@%s\n' % \
                     ( the_password, the_host_port, the_login_name, the_host_name ) )
    else :
        print_d( 'ssh -o "StrictHostKeyChecking no" -i %s -p %d %s@%s\n' % \
                     ( the_identity_file, the_host_port, the_login_name, the_host_name ) )
        pass

    pass


#--------------------------------------------------------------------------------------
def compose_call( the_options ) :
    a_password, an_identity_file, a_host_port, a_login_name, a_host_name, a_command = unpack( the_options )

    if a_password != "" :
        a_call = "--password='%s' --host-port=%d --login-name='%s' --host-name='%s' --command='%s'" % \
            ( a_password, a_host_port, a_login_name, a_host_name, a_command )
    else :
        a_call = "--identity-file='%s' --host-port=%d --login-name='%s' --host-name='%s' --command='%s'" % \
            ( an_identity_file, a_host_port, a_login_name, a_host_name, a_command )
        pass
    
    return a_call


#--------------------------------------------------------------------------------------
def extract_options( the_options ) :
    a_password = the_options.password
    an_identity_file = the_options.identity_file

    if a_password == "" and an_identity_file == "" :
        a_password = raw_input()
        an_identity_file = raw_input()
        pass
    the_options.password = a_password

    if an_identity_file != "" :
        import os.path
        an_identity_file = os.path.expanduser( an_identity_file )
        an_identity_file = os.path.abspath( an_identity_file )
        pass
    the_options.identity_file = an_identity_file

    a_host_port = the_options.host_port
    if a_host_port == None :
        a_host_port = int( raw_input() )
        pass
    the_options.host_port = a_host_port

    a_login_name = the_options.login_name
    if a_login_name == None :
        a_login_name = raw_input()
        pass
    the_options.login_name = a_login_name

    a_host_name = the_options.host_name
    if a_host_name == None :
        a_host_name = raw_input()
        pass
    the_options.host_name = a_host_name

    a_command = the_options.command

    return a_password, an_identity_file, a_host_port, a_login_name, a_host_name, a_command


#--------------------------------------------------------------------------------------
def print_options( the_password, the_identity_file, the_host_port, the_login_name, the_host_name, the_command = None ) :
    print the_password
    print the_identity_file
    print the_host_port
    print the_login_name
    print the_host_name

    pass


#---------------------------------------------------------------------------
def command( the_ssh_client, the_command ) :
    "Execution of secure shell command"
    print_d( "[%s]\n" % the_command )
    
    stdin, stdout, stderr = the_ssh_client.exec_command( the_command )

    a_stderr_lines = stderr.readlines()
    for a_line in a_stderr_lines : print_d( "\t%s" % a_line )

    a_stdout_lines = stdout.readlines()
    for a_line in a_stdout_lines : print_d( "\t%s" % a_line )

    return a_stdout_lines


#--------------------------------------------------------------------------------------
def wait( the_ssh_connect, the_ssh_client, the_command ) :
    print_d( "ssh'ing " )
    while True :
        try :
            print_d( '.' )
            the_ssh_connect()
            command( the_ssh_client, the_command )
            break
        except :
            # import sys, traceback
            # traceback.print_exc( file = sys.stderr )
            continue
        pass

    pass


#--------------------------------------------------------------------------------------
def connect( the_password, the_identity_file, the_host_port, the_login_name, the_host_name, the_command = 'echo  > /dev/null' ) :
    import paramiko
    a_ssh_client = paramiko.SSHClient()
    a_ssh_client.set_missing_host_key_policy( paramiko.AutoAddPolicy() )

    a_ssh_connect = None
    if the_password != "" :
        a_ssh_connect = lambda : a_ssh_client.connect( hostname = the_host_name, port = the_host_port, username = the_login_name, password = the_password )
    else :
        a_rsa_key = paramiko.RSAKey( filename = the_identity_file )
        a_ssh_connect = lambda : a_ssh_client.connect( hostname = the_host_name, port = the_host_port, username = the_login_name, pkey = a_rsa_key )
        pass
    
    wait( a_ssh_connect, a_ssh_client, the_command ) 
    
    return a_ssh_client


#--------------------------------------------------------------------------------------



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script helps to perform a command into remote cloud instance through ssh protocol,
  where the main problem is to establish proper connection 
  (server inocation & ssh timeout problems)
"""


#--------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_e, sh_command, Timer
from balloon.common import ssh


#--------------------------------------------------------------------------------------
# Defining utility command-line interface

an_usage_description = "%prog [ --script-file='./remote_adjust_profile.sh|./remote_update_sources-list.sh' ] [ --sequence-file='./remote_sshd-config.sh' ]"
an_usage_description += ssh.add_usage_description()
an_usage_description += common.add_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
an_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

an_option_parser.add_option( "--script-file",
                             metavar = "< script (or list of scripts separated by '|') to be executed on the remote host >",
                             action = "store",
                             dest = "script_file",
                             default = None )
an_option_parser.add_option( "--script-args",
                             metavar = "< arguments (or list of arguments separated by '|') for the remote script execution >",
                             action = "store",
                             dest = "script_args",
                             default = "" )
an_option_parser.add_option( "--sequence-file",
                             metavar = "< file with sequence of commands to be executed >",
                             action = "store",
                             dest = "sequence_file",
                             default = None )
ssh.add_parser_options( an_option_parser )
common.add_parser_options( an_option_parser )
  
 
#--------------------------------------------------------------------------------------
# Extracting and verifying command-line arguments

an_options, an_args = an_option_parser.parse_args()

an_enable_debug = common.extract_options( an_options )
a_password, an_identity_file, a_host_port, a_login_name, a_host_name, a_command = ssh.extract_options( an_options )

import sys
an_engine = sys.argv[ 0 ]

a_call = "%s %s" % ( an_engine, ssh.compose_call( an_options ) )

import os.path

an_args = None
a_scripts = None
a_script_file = an_options.script_file
if a_script_file != None :
    # First, check that all appointed scripts exists and regular files
    a_scripts = a_script_file.split( '|' )
    for a_script in a_scripts :
        if not os.path.isfile( a_script ) :
            an_option_parser.error( "--script-file='%s' must be a file" % a_script )
            pass
        pass

    # Next, check wether number of 'scripts' match number of script's args
    a_script_args = an_options.script_args
    if a_script_args != "" :
        an_args = a_script_args.split( '|' )
        if len( an_args ) != len( a_scripts ) :
            an_option_parser.error( "number of items in --script-file='%s'"
                                   " must the same or zero as for --script-args='%s' " 
                                   % ( a_script_files, a_script_args ) )
            pass
    else :
        an_args = [ "" for a_script in a_scripts ]
        pass
    
    a_call += " --script-file='%s'" % a_script_file
    if a_script_args != "" :
        a_call += " --script-args='%s'" % a_script_args
        pass
    pass

a_sequence_file = an_options.sequence_file
if a_sequence_file != None :
    a_sequence_file = os.path.abspath( a_sequence_file )
    if not os.path.isfile( a_sequence_file ) :
        an_option_parser.error( "--sequence-file='%s' must be a file" % a_sequence_file )
        pass
    a_call += " --sequence-file='%s'" % a_sequence_file
    pass


print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
print_d( a_call + '\n' )
ssh.print_call( a_password, an_identity_file, a_host_port, a_login_name, a_host_name )


print_d( "\n----------------------- Running actual functionality ----------------------\n" )
a_ssh_client = ssh.connect( a_password, an_identity_file, a_host_port, a_login_name, a_host_name )

if a_scripts != None :
    for an_id in range( len( a_scripts ) ) :
        a_script_file = a_scripts[ an_id ]
        a_script_args = an_args[ an_id ]
        
        a_working_dir = ssh.command( a_ssh_client, 'python -c "import os, os.path, tempfile; print tempfile.mkdtemp()"' )[ 0 ][ : -1 ]
        a_target_script = os.path.join( a_working_dir, os.path.basename( a_script_file ) )
        
        a_sftp_client = a_ssh_client.open_sftp() # Instantiating a sftp client
        a_sftp_client.put( a_script_file, a_target_script )
        
        ssh.command( a_ssh_client, 'chmod 755 "%s"' % a_target_script )
        ssh.command( a_ssh_client, 'sudo "%s" %s' % ( a_target_script, a_script_args ) )

        # ssh.command( a_ssh_client, """python -c 'import shutil; shutil.rmtree( "%s" )'""" % a_working_dir )
        pass
    pass


if a_sequence_file != None :
    a_file = open( a_sequence_file )
    for a_line in a_file.readlines() :
        if a_line[ 0 ] == "#" or a_line[ 0 ] == "\n" :
            continue
        ssh.command( a_ssh_client, 'sudo %s' % a_line[ : -1 ] )
        pass
    a_file.close()
    pass

a_ssh_client.close()


print_d( "\n------------------ Printing succussive pipeline arguments -----------------\n" )
ssh.print_options( a_password, an_identity_file, a_host_port, a_login_name, a_host_name )


print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
ssh.print_call( a_password, an_identity_file, a_host_port, a_login_name, a_host_name )
print_d( a_call + '\n' )


print_d( "\n-------------------------------------- OK ---------------------------------\n" )



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
Lists the all avaialble Amazon AWS locations
"""


#--------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_e, sh_command, Timer
from balloon.common import ssh

from balloon import amazon
from balloon.amazon import ec2


#--------------------------------------------------------------------------------------
# Defining utility command-line interface

an_usage_description = "%prog"
an_usage_description += amazon.add_usage_description()
an_usage_description += common.add_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
an_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

amazon.add_parser_options( an_option_parser )
common.add_parser_options( an_option_parser )
  
 
#--------------------------------------------------------------------------------------
# Extracting and verifying command-line arguments

an_options, an_args = an_option_parser.parse_args()

an_enable_debug = common.extract_options( an_options )
AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.extract_options( an_options )


print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
import sys; an_engine = sys.argv[ 0 ]

a_call = "%s" % ( an_engine )
print_d( a_call + '\n' )


print_d( "\n----------------------- Running actual functionality ----------------------\n" )
a_spent_time = Timer()

import boto.ec2
for a_region in boto.ec2.regions( aws_access_key_id = AWS_ACCESS_KEY_ID, aws_secret_access_key = AWS_SECRET_ACCESS_KEY ) :
    print a_region.name
    
    pass
  
print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
print_d( a_call + '\n' )


print_d( "\n-------------------------------------- OK ---------------------------------\n" )



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#------------------------------------------------------------------------------------------
"""
This script is responsible for efficient uploading of multi file data
"""


#------------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_i, print_e, sh_command
from balloon.common import Timer, WorkerPool, compute_md5

import balloon.amazon as amazon
from balloon.amazon.s3 import generate_uploading_dir
from balloon.amazon.s3 import TRootObject, TStudyObject, TFileObject, TSeedObject

import sys, os, os.path, uuid, hashlib


#------------------------------------------------------------------------------------------
def upload_file( the_worker_pool, the_file_path, the_file_location, the_study_object, the_upload_seed_size, the_printing_depth ) :
    a_working_dir = generate_uploading_dir( the_file_path )

    import shutil
    shutil.rmtree( a_working_dir, True )
    os.makedirs( a_working_dir )
    print_d( "a_working_dir = '%s'\n" % a_working_dir, the_printing_depth )

    a_file_dirname = os.path.dirname( the_file_path )
    a_file_basename = os.path.basename( the_file_path )

    import tempfile
    a_tmp_file = tempfile.mkstemp( dir = a_working_dir )[ 1 ]
    # a_tmp_file = tempfile.mkstemp()[ 1 ] # use this work arround for FAT file systems
    sh_command( "cd '%s' &&  tar -czf %s '%s'" % 
                ( a_file_dirname, a_tmp_file, a_file_basename ), the_printing_depth )

    a_statinfo = os.stat( a_tmp_file )
    print_d( "a_statinfo.st_size = %d, bytes\n" % a_statinfo.st_size, the_printing_depth )

    import math
    a_suffix_length = math.log10( float( a_statinfo.st_size ) / the_upload_seed_size )
    if a_suffix_length > 0 :
        a_suffix_length = int( a_suffix_length + 1.0 )
    else:
        a_suffix_length = 0
        pass
    print_d( "a_suffix_length = %d, digits\n" % a_suffix_length, the_printing_depth )

    a_file_seed_target = os.path.join( a_working_dir, a_file_basename )
    sh_command( "cat '%s' | split --bytes=%d --numeric-suffixes --suffix-length=%d - %s.tgz-" % 
                ( a_tmp_file, the_upload_seed_size, a_suffix_length, a_file_seed_target ), the_printing_depth )

    a_file_pointer = open( a_tmp_file, 'rb' )
    a_md5 = compute_md5( a_file_pointer )
    a_hex_md5, a_base64md5 = a_md5

    a_file_pointer.close()
    os.remove( a_tmp_file )

    a_file_object = TFileObject.create( the_study_object, the_file_path, the_file_location, a_hex_md5 )
    print_d( "a_file_object = %s\n" % a_file_object, the_printing_depth )

    pass


#------------------------------------------------------------------------------------------
def upload_files( the_file2locations, the_study_object, the_upload_seed_size, the_printing_depth ) :
    a_worker_pool = WorkerPool( len( the_file2locations ) )

    for a_file, a_location_file in the_file2locations.iteritems() :
        a_worker_pool.charge( upload_file, ( a_worker_pool, a_file, a_location_file, the_study_object, 
                                             the_upload_seed_size, the_printing_depth ) )

        pass

    a_worker_pool.shutdown()
    a_worker_pool.join()
    
    pass


#------------------------------------------------------------------------------------------
def extract_locations( the_file_locations ):

    return the_file_locations.split( common.arg_list_separator() )


#--------------------------------------------------------------------------------------
# Defining utility command-line interface

an_usage_description = "%prog"
an_usage_description += " --study-name='my favorite study' --upload-item-size=5160 --socket-timeout=3"
an_usage_description += " --file-locations= '<file1 location>,<file2 location>,...'"
an_usage_description += common.add_usage_description()
an_usage_description += amazon.add_usage_description()
an_usage_description += amazon.add_timeout_usage_description()
an_usage_description += " <file 1> <file 2> ..."

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
an_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

# Definition of the command line arguments
an_option_parser.add_option( "--study-name",
                             metavar = "< an unique name of the user study >",
                             action = "store",
                             dest = "study_name",
                             help = "(generated, by default)",
                             default = 'tmp-' + str( uuid.uuid4() ) )
an_option_parser.add_option( "--file-locations",
                             metavar = "< location of files inside of the study >",
                             action = "store",
                             dest = "file_locations",
                             help = "(\"%default\", by default)",
                             default = '' )
an_option_parser.add_option( "--upload-item-size",
                             metavar = "< size of file pieces to be uploaded, in bytes >",
                             type = "int",
                             action = "store",
                             dest = "upload_seed_size",
                             help = "(\"%default\", by default)",
                             default = 65536 )
common.add_parser_options( an_option_parser )
amazon.add_parser_options( an_option_parser )
amazon.add_timeout_options( an_option_parser )
    
an_engine_dir = os.path.abspath( os.path.dirname( sys.argv[ 0 ] ) )


#------------------------------------------------------------------------------------------
# Extracting and verifying command-line arguments

an_options, an_args = an_option_parser.parse_args()

common.extract_options( an_options )

a_source_files = None
if len( an_args ) == 0 :
    a_source_files = [ raw_input() ]
else :
    a_source_files = an_args
    pass

a_files = list()
for a_file in a_source_files :
    if not os.path.exists( a_file ) :
        an_option_parser.error( "The given file should exists\n" )
        pass
    a_files.append( os.path.abspath( a_file ) )
    pass

if len( a_files ) == 0 :
    an_option_parser.error( "You should define one valid 'file' at least\n" )
    pass

print_d( "a_files = %r\n" % a_files )

a_file_locations = extract_locations( an_options.file_locations )

a_file2locations = {}
if len( a_file_locations ) > 1 :
    if len( a_files ) != len( a_file_locations ) :
        an_option_parser.error( "The amount of file locations shoudl be equal to the number of given files\n" )
    else :
        for an_id in range( len( a_files ) ) :
            a_file = a_files[ an_id ]
            a_location = a_file_locations[ an_id ]
            a_file2locations[ a_file ] = a_location
            pass
        pass
else :
    a_location = a_file_locations[ 0 ]
    for a_file in a_files:
        a_file2locations[ a_file ] = a_location
        pass
    pass   

print_d( "a_file2locations = %r\n" % a_file2locations )

a_study_name = an_options.study_name
print_d( "a_study_name = '%s'\n" % a_study_name )
    
AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.extract_options( an_options )

amazon.extract_timeout_options( an_options, an_option_parser )


print_i( "--------------------------- Defining the study object ---------------------------\n" )
a_root_object = TRootObject.get( AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )
print_d( "a_root_object = %s\n" % a_root_object )

a_study_object = TStudyObject.create( a_root_object, a_study_name )
print_d( "a_study_object = %s\n" % a_study_object )


print_i( "---------------------------- Uploading study files ------------------------------\n" )
a_data_loading_time = Timer()

upload_files( a_file2locations, a_study_object, an_options.upload_seed_size, 0 )

print_d( "a_data_loading_time = %s, sec\n" % a_data_loading_time )


print_i( "-------------------------------------- OK ---------------------------------------\n" )
print a_study_name


#------------------------------------------------------------------------------------------



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script is responsible for the task packaging and sending it for execution in a cloud
"""


#--------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_e, sh_command, Timer
from balloon.common import ssh

from balloon import amazon
from balloon.amazon import ec2


#--------------------------------------------------------------------------------------
# Defining utility command-line interface

an_usage_description = "%prog"
an_usage_description += ec2.use.add_usage_description()
an_usage_description += amazon.add_usage_description()
an_usage_description += common.add_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
an_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

ec2.use.add_parser_options( an_option_parser )
amazon.add_parser_options( an_option_parser )
common.add_parser_options( an_option_parser )
  
 
#--------------------------------------------------------------------------------------
# Extracting and verifying command-line arguments

an_options, an_args = an_option_parser.parse_args()

an_enable_debug = common.extract_options( an_options )
AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.extract_options( an_options )

an_image_location = an_options.image_location
a_reservation_id = an_options.reservation_id
an_identity_file = an_options.identity_file
a_host_port = an_options.host_port
a_login_name = an_options.login_name


print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
import sys
an_engine = sys.argv[ 0 ]

a_call = "%s %s" % ( an_engine, ec2.use.compose_call( an_options ) )
print_d( a_call + '\n' )


print_d( "\n----------------------- Running actual functionality ----------------------\n" )
a_spent_time = Timer()

import boto.ec2
a_target_reservation = None
for a_region in boto.ec2.regions( aws_access_key_id = AWS_ACCESS_KEY_ID, aws_secret_access_key = AWS_SECRET_ACCESS_KEY ) :
    if an_image_location != None and a_region.name != an_image_location :
        continue
    
    print_d( 'a_region = "%s"\n' % a_region.name )

    an_ec2_conn = a_region.connect()
    for a_reservation in an_ec2_conn.get_all_instances() :
        if a_reservation.id == a_reservation_id or a_reservation_id == None :
            an_instance = a_reservation.instances[ 0 ]
            a_status = an_instance.update()
            if a_status == 'terminated' :
                continue

            a_target_reservation = a_reservation
            a_reservation_id = a_reservation.id

            an_image_location = a_region.name

            if an_identity_file == None :
                import os.path; a_key_pair_dir = os.path.expanduser( "~/.ssh")
                an_identity_file = os.path.join( a_key_pair_dir, an_instance.key_name ) + os.path.extsep + "pem"
                pass
            
            break
        pass

    if a_target_reservation != None :
        break

    pass


if a_target_reservation == None :
    print_d( "\n-------------------- There are no running reservations --------------------\n" )
    import sys, os; sys.exit( os.EX_UNAVAILABLE )
    pass

  
print_d( "\n------------------ Printing succussive pipeline arguments -----------------\n" )
ec2.use.print_options( an_image_location, a_reservation_id, an_identity_file, a_host_port, a_login_name )


print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
print_d( a_call + '\n' )


print_d( "\n-------------------------------------- OK ---------------------------------\n" )



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script is responsible for the task packaging and sending it for execution in a cloud
"""


#--------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_list, print_e, sh_command, wait_ssh, Timer

import balloon.rackspace as rackspace

import sys, os, os.path, uuid


#--------------------------------------------------------------------------------------
# Defining utility command-line interface

an_usage_description = "%prog"
an_usage_description += common.add_usage_description()
an_usage_description += rackspace.add_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
an_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

rackspace.add_parser_options( an_option_parser )
common.add_parser_options( an_option_parser )
    

#--------------------------------------------------------------------------------------
# Extracting and verifying command-line arguments

an_options, an_args = an_option_parser.parse_args()

an_enable_debug = common.extract_options( an_options )
RACKSPACE_USER, RACKSPACE_KEY = rackspace.extract_options( an_options )


print_d( "\n----------------------- Running actual functionality ----------------------\n" )
an_instance_reservation_time = Timer()

from libcloud.types import Provider 
from libcloud.providers import get_driver 

Driver = get_driver( Provider.RACKSPACE ) 
a_libcloud_conn = Driver( RACKSPACE_USER, RACKSPACE_KEY ) 
print_d( "a_libcloud_conn = %r\n" % a_libcloud_conn )

an_images = a_libcloud_conn.list_images() 
print_list( "an_images :\n" , an_images )

a_sizes = a_libcloud_conn.list_sizes() 
print_list( "a_sizes :\n" , a_sizes )


print_d( "\n-------------------------------------- OK ---------------------------------\n" )



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#------------------------------------------------------------------------------------------
"""
This script is responsible for efficient uploading of multi file data
"""


#------------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_i, print_e, sh_command
from balloon.common import Timer, WorkerPool, compute_md5

import balloon.amazon as amazon
from balloon.amazon.s3 import generate_uploading_dir
from balloon.amazon.s3 import TRootObject, TStudyObject, TFileObject, TSeedObject

import sys, os, os.path, uuid, hashlib


#------------------------------------------------------------------------------------------
def mark_finished( the_file_object, the_working_dir, the_printing_depth ) :
    try :
        the_file_object.seal( the_working_dir )

        return True
    except :
        import sys, traceback
        traceback.print_exc( file = sys.stderr )

        pass

    return False


#------------------------------------------------------------------------------------------
def upload_seed( the_file_object, the_seed_name, the_seed_path, the_printing_depth ) :
    "Uploading file item"
    try :
        a_seed_object = TSeedObject.create( the_file_object, the_seed_name, the_seed_path )
        print_d( "%s\n" % a_seed_object, the_printing_depth )
        
        return True
    except :
        import sys, traceback
        traceback.print_exc( file = sys.stderr )

        pass

    return False


#------------------------------------------------------------------------------------------
def upload_seeds( the_file_object, the_working_dir, the_number_threads, the_printing_depth ) :
    "Uploading file items"
    while True :
        a_dir_contents = os.listdir( the_working_dir )

        a_number_threads = max( min( the_number_threads, len( a_dir_contents ) ), 1 )
        print_d( "a_number_threads = %d\n" % a_number_threads, the_printing_depth )

        a_worker_pool = WorkerPool( a_number_threads )
        
        a_dir_contents.sort()
        a_dir_contents.reverse()
        
        a_seed_names = [ a_seed_object.name() for a_seed_object in the_file_object ]
        
        for a_seed_name in a_dir_contents :
            a_seed_path = os.path.join( the_working_dir, a_seed_name )
            print_d( "'%s'\n" % a_seed_path, the_printing_depth + 1 )
        
            if a_seed_name in a_seed_names :
                os.remove( a_seed_path )

                continue

            a_worker_pool.charge( upload_seed, ( the_file_object, a_seed_name, a_seed_path, the_printing_depth + 2 ) )

            pass

        a_worker_pool.shutdown()
        an_is_all_right = a_worker_pool.is_all_right()

        if an_is_all_right :
            mark_finished( the_file_object, the_working_dir, the_printing_depth )

            break

        pass

    return True


#------------------------------------------------------------------------------------------
def upload_file( the_file_object, the_number_threads, the_printing_depth ) :
    a_working_dir = generate_uploading_dir( the_file_object.file_path() )
    if not os.path.exists( a_working_dir ) :
        return True

    print_d( "a_working_dir = '%s'\n" % a_working_dir, the_printing_depth )

    return upload_seeds( the_file_object, a_working_dir, the_number_threads, the_printing_depth + 1 )


#------------------------------------------------------------------------------------------
def upload_files( the_study_object, the_number_threads, the_printing_depth ) :
    for a_file_object in the_study_object :
        print_d( "a_file_object = %s\n" % a_file_object, the_printing_depth )

        upload_file( a_file_object, the_number_threads, the_printing_depth + 1 )
        
        pass

    pass


#------------------------------------------------------------------------------------------
# Defining utility command-line interface

an_usage_description = "%prog --study-name='my interrupted study'"
an_usage_description += common.add_usage_description()
an_usage_description += amazon.add_usage_description()
an_usage_description += amazon.add_timeout_usage_description()
an_usage_description += amazon.add_threading_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
a_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

# Definition of the command line arguments
a_option_parser.add_option( "--study-name",
                            metavar = "< an unique name of the user study >",
                            action = "store",
                            dest = "study_name" )
common.add_parser_options( a_option_parser )
amazon.add_parser_options( a_option_parser )
amazon.add_timeout_options( a_option_parser )
amazon.add_threading_parser_options( a_option_parser )
    
an_engine_dir = os.path.abspath( os.path.dirname( sys.argv[ 0 ] ) )


#------------------------------------------------------------------------------------------
# Extracting and verifying command-line arguments

an_options, an_args = a_option_parser.parse_args()

common.extract_options( an_options )

a_study_name = an_options.study_name
if a_study_name == None :
    a_study_name = raw_input()
    pass

print_d( "a_study_name = '%s'\n" % a_study_name )
    
AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.extract_options( an_options )

a_number_threads = amazon.extract_threading_options( an_options, a_option_parser )

amazon.extract_timeout_options( an_options, a_option_parser )


print_i( "--------------------------- Looking for study object ----------------------------\n" )
a_root_object = TRootObject.get( AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )
print_d( "a_root_object = %s\n" % a_root_object )

a_study_object = TStudyObject.get( a_root_object, a_study_name )
print_d( "a_study_object = %s\n" % a_study_object )


print_i( "---------------------------- Uploading study files ------------------------------\n" )
a_data_loading_time = Timer()

upload_files( a_study_object, a_number_threads, 0 )

print_d( "a_data_loading_time = %s, sec\n" % a_data_loading_time )


print_i( "-------------------------------------- OK ---------------------------------------\n" )
print a_study_name


#------------------------------------------------------------------------------------------



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
Cleans all nodes from cloudservers and cloudfiles that correspond to defined rackspace account
"""

#--------------------------------------------------------------------------------------
import os
AWS_ACCESS_KEY_ID = os.getenv( "AWS_ACCESS_KEY_ID" )
AWS_SECRET_ACCESS_KEY = os.getenv( "AWS_SECRET_ACCESS_KEY" )


#--------------------------------------------------------------------------------------
import boto

import balloon.common as common
from balloon.common import print_d, print_i, print_e, sh_command, Timer, WorkerPool


#--------------------------------------------------------------------------------------
print "--------------- Delete S3 buckets ----------------"
a_s3_conn = boto.connect_s3( AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )

a_worker_pool = WorkerPool( 8 )

# First remove all the bucket keys
for a_bucket in a_s3_conn.get_all_buckets() :
    try :
        a_s3_bucket_keys = a_bucket.get_all_keys()
        print "'%s' : %d" % ( a_bucket.name, len( a_s3_bucket_keys ) )
        
        for a_s3_bucket_key in a_bucket.list() :
            print "\t'%s'" % ( a_s3_bucket_key.name )
            a_worker_pool.charge( lambda the_s3_bucket_key : the_s3_bucket_key.delete(), [ a_s3_bucket_key ] )
            
            pass
    except :
        pass

    pass

a_worker_pool.join()
print

# Remove the bucket itself
for a_bucket in a_s3_conn.get_all_buckets() :
    print "'%s'" % ( a_bucket.name )
    a_worker_pool.charge( lambda the_s3_bucket : the_s3_bucket.delete(), [ a_bucket ] )

    pass

a_worker_pool.shutdown()
a_worker_pool.join()
print


print "---------------------- OK ------------------------"
#--------------------------------------------------------------------------------------



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#------------------------------------------------------------------------------------------
"""
This script is responsible for efficient downloading of multi file data
"""


#------------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_i, print_e, sh_command
from balloon.common import Timer, WorkerPool, compute_md5

import balloon.amazon as amazon
from balloon.amazon.s3 import TRootObject, TStudyObject, TFileObject, TSeedObject

import boto
from boto.s3.key import Key

import sys, os, os.path, uuid, hashlib


#------------------------------------------------------------------------------------------
def download_seed( the_seed_object, the_file_path, the_printing_depth ) :
    try :
        the_seed_object.download( the_file_path )
        print_d( "the_seed_object = %s\n" % the_seed_object, the_printing_depth )
        
        return True
    except :
        import sys, traceback
        traceback.print_exc( file = sys.stderr )

        pass

    return False


#------------------------------------------------------------------------------------------
def download_seeds( the_file_object, the_file_basename, the_output_dir, the_number_threads, the_printing_depth ) :
    an_is_download_ok = False
    an_is_everything_uploaded = False
    while not an_is_everything_uploaded or not an_is_download_ok :
        a_worker_pool = WorkerPool( the_number_threads )

        for a_seed_object in the_file_object :
            if a_seed_object.is_seal() :
                an_is_everything_uploaded = True
                print_d( "an_is_everything_uploaded = %s\n" % an_is_everything_uploaded, the_printing_depth )
                continue

            a_hex_md5 = a_seed_object.hex_md5()
            a_seed_path = os.path.join( the_output_dir, a_seed_object.name() )

            if os.path.exists( a_seed_path ) :
                a_file_pointer = open( a_seed_path, 'rb' )
                a_md5 = compute_md5( a_file_pointer )[ 0 ]

                if a_hex_md5 == a_md5 :
                    continue

                os.remove( a_seed_path )

                pass

            print_d( "a_seed_path = '%s'\n" % a_seed_path, the_printing_depth )

            a_worker_pool.charge( download_seed, ( a_seed_object, a_seed_path, the_printing_depth + 1 ) )
            
            pass

        a_worker_pool.shutdown()
        an_is_download_ok = a_worker_pool.is_all_right()

        pass
    
    return True


#------------------------------------------------------------------------------------------
def download_file( the_file_object, the_output_dir, the_number_threads, the_enable_fresh, the_printing_depth ) :
    print_d( "the_file_object = %s\n" % the_file_object, the_printing_depth )

    a_hex_md5 = the_file_object.hex_md5()
    a_located_file = the_file_object.located_file()

    a_file_dirname = os.path.dirname( a_located_file )
    a_file_basename = os.path.basename( a_located_file )

    an_output_dir = os.path.join( the_output_dir, a_file_dirname )
    print_d( "an_output_dir = '%s'\n" % an_output_dir, the_printing_depth + 1 )
    
    
    a_file_path = os.path.join( an_output_dir, a_file_basename )
    if the_enable_fresh :
       import shutil
       shutil.rmtree( a_file_path, True )
       pass

    if not os.path.exists( an_output_dir ) :
        os.makedirs( an_output_dir )
        pass

    
    print_d( "a_file_path = '%s'\n" % a_file_path, the_printing_depth + 2 )
    if os.path.exists( a_file_path ) :
        print_d( "nothing to be done, already downloaded\n", the_printing_depth + 3 )

        return True

    while True :
        download_seeds( the_file_object, a_file_basename, an_output_dir, the_number_threads, the_printing_depth + 3 )
        
        an_archive_name = "%s.tgz" % a_file_basename

        sh_command( "cd '%s' && cat %s-* > %s" % ( an_output_dir, an_archive_name, an_archive_name ), the_printing_depth )

        an_archive_path = os.path.join( an_output_dir, an_archive_name )
        an_archive_pointer = open( an_archive_path, 'rb' )
        a_md5 = compute_md5( an_archive_pointer )[ 0 ]

        print_d( "'%s' - %s\n" % ( a_hex_md5, ( a_hex_md5 == a_md5 ) ), the_printing_depth )

        if a_hex_md5 == a_md5 :
            break

        pass

    sh_command( "tar -xzf '%s' -C '%s'" % ( an_archive_path, an_output_dir ), the_printing_depth + 2 )

    sh_command( "cd '%s' && rm %s-*" % ( an_output_dir, an_archive_name ), the_printing_depth + 2 )

    os.remove( an_archive_path )

    return True


#------------------------------------------------------------------------------------------
def download_files( the_study_object, the_output_dir, the_number_threads, the_enable_fresh, the_printing_depth ) :
    a_worker_pool = WorkerPool( the_number_threads )

    for a_file_object in the_study_object :
        a_worker_pool.charge( download_file, ( a_file_object, the_output_dir, the_number_threads, the_enable_fresh, the_printing_depth ) )
        
        pass

    a_worker_pool.shutdown()
    a_worker_pool.join()

    pass


#------------------------------------------------------------------------------------------
# Defining utility command-line interface

an_usage_description = "%prog"
an_usage_description += " --study-name='my uploaded study' --output-dir='./tmp'"
an_usage_description += " --located-files= '<location-in-study-1/file-1>, <location-in-study-2/file-2>' ... "
an_usage_description += common.add_usage_description()
an_usage_description += amazon.add_usage_description()
an_usage_description += amazon.add_timeout_usage_description()
an_usage_description += amazon.add_threading_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
a_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

# Definition of the command line arguments
a_option_parser.add_option( "--study-name",
                            metavar = "< a name of an uploaded study >",
                            action = "store",
                            dest = "study_name",
                            help = "(intialized from input, otherwise)" )
a_option_parser.add_option( "--output-dir",
                            metavar = "< location of the task defintion >",
                            action = "store",
                            dest = "output_dir",
                            help = "(the same a 'study' name, by default)",
                            default = None )
a_option_parser.add_option( "--located-files",
                            metavar = "< the list of file paths inside the study >",
                            action = "store",
                            dest = "located_files",
                            default = None )
a_option_parser.add_option( "--enable-fresh",
                            action = "store_true",
                            dest = "enable_fresh",
                            help = "do not take into account previous downloads",
                            default = False )
common.add_parser_options( a_option_parser )
amazon.add_parser_options( a_option_parser )
amazon.add_timeout_options( a_option_parser )
amazon.add_threading_parser_options( a_option_parser )
    
an_engine_dir = os.path.abspath( os.path.dirname( sys.argv[ 0 ] ) )


#------------------------------------------------------------------------------------------
# Extracting and verifying command-line arguments

an_options, an_args = a_option_parser.parse_args()

common.extract_options( an_options )

a_study_name = an_options.study_name
if a_study_name == None :
    a_study_name = raw_input()
    pass

print_d( "a_study_name = '%s'\n" % a_study_name )
    
an_output_dir = an_options.output_dir
if an_output_dir == None :
    an_output_dir = os.path.join( an_engine_dir, a_study_name )
    pass

print_d( "an_output_dir = '%s'\n" % an_output_dir )

an_enable_fresh = an_options.enable_fresh
print_d( "an_enable_fresh = %s\n" % an_enable_fresh )

AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.extract_options( an_options )

a_number_threads = amazon.extract_threading_options( an_options, a_option_parser )

print_d( "a_number_threads = %d\n" % a_number_threads )

amazon.extract_timeout_options( an_options, a_option_parser )


print_i( "--------------------------- Looking for study object ----------------------------\n" )
a_root_object = TRootObject.get( AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )
print_d( "a_root_object = %s\n" % a_root_object )

a_study_object = TStudyObject.get( a_root_object, a_study_name )
print_d( "a_study_object = %s\n" % a_study_object )


print_i( "--------------------------- Reading the study files -----------------------------\n" )
a_data_loading_time = Timer()

a_located_files = an_options.located_files

if a_located_files == None :
    download_files( a_study_object, an_output_dir, a_number_threads, an_enable_fresh, 0 )
    pass
else :
    a_located_files = a_located_files.split( common.arg_list_separator() )
      
    a_worker_pool = WorkerPool( a_number_threads )
      
    for a_located_file in a_located_files:
        a_file_object = TFileObject.get( a_study_object, a_located_file )
        a_worker_pool.charge( download_file, ( a_file_object, an_output_dir, a_number_threads, an_enable_fresh, 0 ) )                    
        pass
    
    a_worker_pool.shutdown()
    a_worker_pool.join()
    pass
    
print_d( "a_data_loading_time = %s, sec\n" % a_data_loading_time )


print_d( "\n------------------ Printing succussive pipeline arguments -----------------\n" )
print a_study_name


print_i( "-------------------------------------- OK ---------------------------------------\n" )




#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script is responsible for the deployng of the 'balloon' package into appointed cloud instance
"""


#--------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_e, sh_command, Timer
from balloon.common import ssh


#--------------------------------------------------------------------------------------
# Defining utility command-line interface

an_usage_description = "%prog"
an_usage_description += ssh.add_usage_description()
an_usage_description += common.add_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
a_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

ssh.add_parser_options( a_option_parser )
common.add_parser_options( a_option_parser )
  
 
#--------------------------------------------------------------------------------------
# Extracting and verifying command-line arguments

an_options, an_args = a_option_parser.parse_args()

an_enable_debug = common.extract_options( an_options )
a_password, an_identity_file, a_host_port, a_login_name, a_host_name, a_command = ssh.extract_options( an_options )

print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
import sys
an_engine = sys.argv[ 0 ]

a_call = "%s %s" % ( an_engine, ssh.compose_call( an_options ) )

print_d( a_call + '\n' )
ssh.print_call( a_password, an_identity_file, a_host_port, a_login_name, a_host_name )


print_d( "\n----------------------- Running actual functionality ----------------------\n" )
import os.path
an_engine_dir = os.path.abspath( os.path.dirname( an_engine ) )
sh_command( "cd %s && ./setup.py sdist" % an_engine_dir )

a_ssh_client = ssh.connect( a_password, an_identity_file, a_host_port, a_login_name, a_host_name )

ssh.command( a_ssh_client, ( "sudo apt-get -y install python-setuptools" ) )
ssh.command( a_ssh_client, ( "sudo apt-get -y install python-all-dev" ) )

a_working_dir = ssh.command( a_ssh_client, 'python -c "import os, os.path, tempfile; print tempfile.mkdtemp()"' )[ 0 ][ : -1 ]
print_d( "a_working_dir = %s\n" % a_working_dir )

import balloon
a_balloon_name = "%s-%s" % ( balloon.NAME, balloon.VERSION )
a_balloon_archive_name = a_balloon_name + os.extsep + "tar.gz"
a_balloon_source_archive = os.path.join( an_engine_dir, 'dist', a_balloon_archive_name )
a_balloon_target_archive = os.path.join( a_working_dir, a_balloon_archive_name )

# Uploading and installing into the cloud corresponding Python engine (itself)
a_sftp_client = a_ssh_client.open_sftp()
a_sftp_client.put( a_balloon_source_archive, a_balloon_target_archive )
ssh.command( a_ssh_client, 'cd %s && tar -xzf %s' % ( a_working_dir, a_balloon_archive_name ) )
a_balloon_setup_dir = os.path.join( a_working_dir, a_balloon_name )
ssh.command( a_ssh_client, 'cd %s && sudo python ./setup.py install' % ( a_balloon_setup_dir ) )

# ssh.command( a_ssh_client, """python -c 'import shutil; shutil.rmtree( "%s" )'""" % a_working_dir )
a_ssh_client.close()


print_d( "\n------------------ Printing succussive pipeline arguments -----------------\n" )
ssh.print_options( a_password, an_identity_file, a_host_port, a_login_name, a_host_name )


print_d( "\n--------------------------- Canonical substitution ------------------------\n" )
ssh.print_call( a_password, an_identity_file, a_host_port, a_login_name, a_host_name )
print_d( a_call + '\n' )


print_d( "\n-------------------------------------- OK ---------------------------------\n" )




#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script is responsible for fetching of the task resulting data from the cloud
"""


#--------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_e

import balloon.rackspace as rackspace
import balloon.amazon as amazon

import os


#--------------------------------------------------------------------------------------
# Defining utility command-line interface

an_usage_description = "%prog --container-name=33f89d9d-5417-49c1-80c9-787e74cc7154 --output-dir=."
an_usage_description += common.add_usage_description()
an_usage_description += rackspace.add_usage_description()
an_usage_description += amazon.add_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
a_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

# Definition of the command line arguments
a_option_parser.add_option( "--container-name",
                            metavar = "< name of task container >",
                            action = "store",
                            dest = "container_name" )

a_option_parser.add_option( "--output-dir",
                            metavar = "< location of the task defintion >",
                            action = "store",
                            dest = "output_dir",
                            help = "(\"%default\", by default)",
                            default = "." )

common.add_parser_options( a_option_parser )

rackspace.add_parser_options( a_option_parser )

amazon.add_parser_options( a_option_parser )
    

#--------------------------------------------------------------------------------------
# Extracting and verifying command-line arguments

an_options, an_args = a_option_parser.parse_args()

common.extract_options( an_options )

a_container_name = an_options.container_name
if a_container_name == None :
    a_container_name = raw_input()
    pass

RACKSPACE_USER, RACKSPACE_KEY = rackspace.extract_options( an_options )

AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.extract_options( an_options )


#---------------------------------------------------------------------------
# Trying to access to the container with the appointed name

import cloudfiles
a_cloudfiles_conn = cloudfiles.get_connection( RACKSPACE_USER, RACKSPACE_KEY, timeout = 500 )
a_cloudfiles_container = a_cloudfiles_conn[ a_container_name ]


#---------------------------------------------------------------------------
# Creating an output directory

import os.path, shutil
an_options.output_dir = os.path.abspath( an_options.output_dir )
an_output_dir = os.path.join( an_options.output_dir, a_container_name )
shutil.rmtree( an_output_dir, True )
os.makedirs( an_output_dir )
if not os.path.isdir( an_output_dir ) :
    print_e( "Couild not create output directory - '%s'\n" % an_output_dir )
    pass


#---------------------------------------------------------------------------
# Downloding the data from cloud according to the queue

import boto
a_sqs_conn = boto.connect_sqs( AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )
a_queue_name = common.generate_initial_queue_name( a_container_name )
while True :
    print_d( '%s\n' % a_queue_name ) 
    an_is_final, a_data_name, a_queue_suffix = common.get_message( a_sqs_conn, a_queue_name )
    print_d( '%s %s\n' % ( a_data_name, a_queue_suffix ) )
    
    a_queue_name = common.generate_queue_name( a_container_name, a_queue_suffix )
    
    # To secure the following 'save' operation
    a_file_path = os.path.join( an_output_dir, a_data_name )
    shutil.rmtree( a_file_path, True ) 
    print_d( '%s ' % ( not os.path.isfile( a_file_path ) ) )
    
    a_file_object = a_cloudfiles_container.get_object( a_data_name )
    a_file_object.save_to_filename( a_file_path )
    print_d( '%s %s\n' % ( a_file_path, os.path.isfile( a_file_path ) ) )
    
    if an_is_final:
        break
    
    pass


#---------------------------------------------------------------------------
print_d( 'OK\n' )


#--------------------------------------------------------------------------------------





#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
from balloon.common import print_e, print_d, Timer

import os, os.path


#--------------------------------------------------------------------------------------
def add_usage_description() :
    return " --image-id='ami-2d4aa444' --image-location='us-east-1' --instance-type='m1.small' --min-count=1 --max-count=1 --user-name='ubuntu'"


#--------------------------------------------------------------------------------------
def add_parser_options( the_option_parser ) :
    the_option_parser.add_option( "--image-id",
                                  metavar = "< Amazon EC2 AMI ID >",
                                  action = "store",
                                  dest = "image_id",
                                  help = "(\"%default\", by default)",
                                  default = "ami-dec135b7" ) # ami-9c5aaff5
    
    the_option_parser.add_option( "--image-location",
                                  metavar = "< location of the AMI >",
                                  action = "store",
                                  dest = "image_location",
                                  help = "(\"%default\", by default)",
                                  default = "us-east-1" )

    the_option_parser.add_option( "--instance-type",
                                  metavar = "< type of the instance in terms of EC2 >",
                                  action = "store",
                                  dest = "instance_type",
                                  help = "(\"%default\", by default)",
                                  default = "c1.xlarge" ) # m1.small
    
    the_option_parser.add_option( "--min-count",
                                  metavar = "< minimum number of instances to start >",
                                  type = "int",
                                  action = "store",
                                  dest = "min_count",
                                  help = "(%default, by default)",
                                  default = 1 )
    
    the_option_parser.add_option( "--max-count",
                                  metavar = "< minimum number of instances to start >",
                                  type = "int",
                                  action = "store",
                                  dest = "max_count",
                                  help = "(%default, by default)",
                                  default = 1 )

    the_option_parser.add_option( "--host-port",
                                  metavar = "< port to be used for ssh >",
                                  type = "int",
                                  action = "store",
                                  dest = "host_port",
                                  default = 22 )
    pass


#--------------------------------------------------------------------------------------
def unpack( the_options ) :
    an_image_id = the_options.image_id
    an_image_location = the_options.image_location
    an_instance_type = the_options.instance_type
    a_min_count = the_options.min_count
    a_max_count = the_options.max_count
    a_host_port = the_options.host_port

    return an_image_id, an_image_location, an_instance_type, a_min_count, a_max_count, a_host_port


#--------------------------------------------------------------------------------------
def compose_call( the_options ) :
    an_image_id, an_image_location, an_instance_type, a_min_count, a_max_count, a_host_port = unpack( the_options )

    a_call = "--image-id='%s' --image-location='%s' --instance-type='%s' --min-count=%d --max-count=%d --host-port=%d" % \
        ( an_image_id, an_image_location, an_instance_type, a_min_count, a_max_count, a_host_port )
    
    return a_call


#--------------------------------------------------------------------------------------
def extract_options( the_options ) :
    an_image_id = the_options.image_id
    an_image_location = the_options.image_location
    an_instance_type = the_options.instance_type

    a_min_count = the_options.min_count
    a_max_count = the_options.max_count
    if a_min_count > a_max_count :
        import math
        print_d( '--min-count=%d > --max-count=%d : --max-count will be corrected to %d' 
                 % ( a_min_count, a_max_count, a_min_count ) )
        the_options.max_count = a_max_count = a_min_count
        pass

    a_host_port = the_options.host_port

    return an_image_id, an_image_location, an_instance_type, a_min_count, a_max_count, a_host_port


#--------------------------------------------------------------------------------------
def wait4activation( the_instance ) :
    while True :
        try :
            print_d( '%s ' % the_instance.update() )
            break
        except :
            continue
        pass
    
    # Making sure that corresponding instances are ready to use
    while True :
        try :
            if the_instance.update() == 'running' :
                break
            print_d( '.' )
        except :
            continue
        pass
    
    print_d( ' %s\n' % the_instance.update() )

    pass


#--------------------------------------------------------------------------------------
def get_reservation( the_ec2_conn, the_reservation_id ) :
    for a_reservation in the_ec2_conn.get_all_instances() :
        if a_reservation.id == the_reservation_id :
            return a_reservation
        pass
    pass


#--------------------------------------------------------------------------------------
def run_reservation( the_image_id, the_image_location, the_instance_type, 
                     the_min_count, the_max_count, the_host_port,
                     the_aws_access_key_id, the_aws_secret_access_key ) :
    print_d( "\n-------------------------- Defining image location ------------------------\n" )
    an_instance_reservation_time = Timer()

    from common import region_connect
    an_ec2_conn = region_connect( the_image_location, the_aws_access_key_id, the_aws_secret_access_key )

    an_images = an_ec2_conn.get_all_images( image_ids = the_image_id )
    an_image = an_images[ 0 ]
    print_d( 'an_image = < %s >\n' % an_image )


    print_d( "\n---------------- Creating unique key-pair and security group --------------\n" )
    import tempfile
    an_unique_file = tempfile.mkstemp()[ 1 ]
    an_unique_name = os.path.basename( an_unique_file )
    os.remove( an_unique_file )
    print_d( "an_unique_name = '%s'\n" % an_unique_name )

    # Asking EC2 to generate a new ssh "key pair"
    a_key_pair_name = an_unique_name
    a_key_pair = an_ec2_conn.create_key_pair( a_key_pair_name )
    
    # Saving the generated ssh "key pair" locally
    a_key_pair_dir = os.path.expanduser( "~/.ssh")
    if not os.path.exists( a_key_pair_dir ) :
        os.makedirs( a_key_pair_dir )
        pass
    a_key_pair.save( a_key_pair_dir )
    an_identity_file = os.path.join( a_key_pair_dir, a_key_pair.name ) + os.path.extsep + "pem"
    print_d( "an_identity_file = '%s'\n" % an_identity_file )

    import stat
    os.chmod( an_identity_file, stat.S_IRUSR )

    # Asking EC2 to generate a new "sequirity group" & apply corresponding firewall permissions
    a_security_group = an_ec2_conn.create_security_group( an_unique_name, 'temporaly generated' )
    a_security_group.authorize( 'tcp', the_host_port, the_host_port, '0.0.0.0/0' ) # ssh port
    
    
    print_d( "\n-------------------------------- Running image ----------------------------\n" )
    # Creating a EC2 "reservation" with all the parameters mentioned above
    a_reservation = an_image.run( instance_type = the_instance_type, min_count = the_min_count, max_count = the_max_count, 
                                  key_name = a_key_pair_name, security_groups = [ a_security_group.name ] )
    print_d( '< %r > : %s\n' % ( a_reservation, a_reservation.instances ) )

    for an_instance in a_reservation.instances :
        wait4activation( an_instance ) # Making sure that corresponding instances are ready to use
        pass

    return a_reservation, an_identity_file


#--------------------------------------------------------------------------------------



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
Cleans all nodes from cloudservers and cloudfiles that correspond to defined rackspace account
"""

#--------------------------------------------------------------------------------------
import os
RACKSPACE_USER = os.getenv( "RACKSPACE_USER" )
RACKSPACE_KEY = os.getenv( "RACKSPACE_KEY" )


#--------------------------------------------------------------------------------------
import cloudfiles
print "---------------- Delete CloudFiles ---------------"
a_cloudfiles_conn = cloudfiles.get_connection( RACKSPACE_USER, RACKSPACE_KEY, timeout = 500 )
for a_container_name in a_cloudfiles_conn.list_containers() :
    if a_container_name == 'cloudservers' :
        continue

    print a_container_name
    a_cloudfiles_container = a_cloudfiles_conn[ a_container_name ]
    an_objects = a_cloudfiles_container.get_objects()
    for an_object in an_objects :
        a_cloudfiles_container.delete_object( an_object )
        print "\t", an_object
        pass

    a_cloudfiles_conn.delete_container( a_cloudfiles_container ) 
    pass

print


#--------------------------------------------------------------------------------------
from libcloud.types import Provider 
from libcloud.providers import get_driver 
print "-------------- Destroy CloudServers --------------"
Driver = get_driver( Provider.RACKSPACE ) 
a_libcloud_conn = Driver( RACKSPACE_USER, RACKSPACE_KEY ) 
for a_node in a_libcloud_conn.list_nodes() :
    a_node.destroy()
    print a_node.name
    pass

print


#--------------------------------------------------------------------------------------
print "---------------------- OK ------------------------"


#--------------------------------------------------------------------------------------





#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
from balloon.common import print_e, print_d

import os


#--------------------------------------------------------------------------------------
def add_usage_description() :
    return " --aws-access-key-id=${AWS_ACCESS_KEY_ID} --aws-secret-access-key=${AWS_SECRET_ACCESS_KEY}"


#--------------------------------------------------------------------------------------
def add_parser_options( the_option_parser ) :
    the_option_parser.add_option( "--aws-access-key-id",
                                  metavar = "< Amazon key id >",
                                  action = "store",
                                  dest = "aws_access_key_id",
                                  help = "(${AWS_ACCESS_KEY_ID}, by default)",
                                  default = os.getenv( "AWS_ACCESS_KEY_ID" ) )
    
    the_option_parser.add_option( "--aws-secret-access-key",
                                  metavar = "< Amazon secret key >",
                                  action = "store",
                                  dest = "aws_secret_access_key",
                                  help = "(${AWS_SECRET_ACCESS_KEY}, by default)",
                                  default = os.getenv( "AWS_SECRET_ACCESS_KEY" ) )
    pass


#--------------------------------------------------------------------------------------
def unpack( the_options ) :
    AWS_ACCESS_KEY_ID = the_options.aws_access_key_id
    AWS_SECRET_ACCESS_KEY = the_options.aws_secret_access_key

    return AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY


#--------------------------------------------------------------------------------------
def compose_call( the_options ) :
    AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = unpack( the_options )

    a_call = "--aws-access-key-id=${AWS_ACCESS_KEY_ID} --aws-secret-access-key=${AWS_SECRET_ACCESS_KEY}"
    
    return a_call


#--------------------------------------------------------------------------------------
def extract_options( the_options ) :
    AWS_ACCESS_KEY_ID = the_options.aws_access_key_id
    if AWS_ACCESS_KEY_ID == None :
        print_e( "Define AWS_ACCESS_KEY_ID parameter through '--aws-access-key-id' option\n" )
        pass

    AWS_SECRET_ACCESS_KEY = the_options.aws_secret_access_key
    if AWS_SECRET_ACCESS_KEY == None :
        print_e( "Define AWS_SECRET_ACCESS_KEY parameter through '--aws-secret-access-key' option\n" )
        pass

    return AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY


#--------------------------------------------------------------------------------------
def add_threading_usage_description() :
    return " --number-threads=7"


#--------------------------------------------------------------------------------------
def add_threading_parser_options( the_option_parser ) :
    the_option_parser.add_option( "--number-threads",
                                  metavar = "< number of threads to use >",
                                  type = "int",
                                  action = "store",
                                  dest = "number_threads",
                                  help = "(\"%default\", by default)",
                                  default = 2 )
    pass


#--------------------------------------------------------------------------------------
def extract_threading_options( the_options, the_option_parser ) :
    if the_options.number_threads < 1 :
        the_option_parser.error( "'--number-threads' must be at least 1" )
        pass

    return the_options.number_threads


#--------------------------------------------------------------------------------------
def add_timeout_usage_description() :
    return " --socket-timeout=3"


#--------------------------------------------------------------------------------------
def add_timeout_options( the_option_parser ) :
    the_option_parser.add_option( "--socket-timeout",
                                  metavar = "< socket timeout time >",
                                  type = "int",
                                  action = "store",
                                  dest = "socket_timeout",
                                  help = "(\"%default\", by default)",
                                  default = None )
    pass


#--------------------------------------------------------------------------------------
def extract_timeout_options( the_options, the_option_parser ) :
    import socket
    socket.setdefaulttimeout( the_options.socket_timeout )

    return the_options.socket_timeout


#------------------------------------------------------------------------------------------



#!/usr/bin/env python

#------------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#------------------------------------------------------------------------------------------
"""
Removes whole appointed cloud study or just given files from this study
"""

#------------------------------------------------------------------------------------------
import balloon.common as common
from balloon.common import print_d, print_i, print_e, sh_command
from balloon.common import Timer, WorkerPool, compute_md5

import balloon.amazon as amazon
from balloon.amazon.s3 import TRootObject, TStudyObject, TFileObject, TSeedObject

import sys, os, os.path, uuid, hashlib


#------------------------------------------------------------------------------------------
# Defining utility command-line interface
an_usage_description = "%prog --study-name=<my favorite study>"
an_usage_description += common.add_usage_description()
an_usage_description += amazon.add_usage_description()
an_usage_description += amazon.add_timeout_usage_description()
an_usage_description += amazon.add_threading_usage_description()

from optparse import IndentedHelpFormatter
a_help_formatter = IndentedHelpFormatter( width = 127 )

from optparse import OptionParser
an_option_parser = OptionParser( usage = an_usage_description, version="%prog 0.1", formatter = a_help_formatter )

# Definition of the command line arguments
an_option_parser.add_option( "--study-name",
                            metavar = "< an unique name of the user study >",
                            action = "store",
                            dest = "study_name",
                            default = None )
common.add_parser_options( an_option_parser, True )
amazon.add_parser_options( an_option_parser )
amazon.add_timeout_options( an_option_parser )
amazon.add_threading_parser_options( an_option_parser )


#------------------------------------------------------------------------------------------
# Extracting and verifying command-line arguments

an_options, an_args = an_option_parser.parse_args()

a_study_name = an_options.study_name
if a_study_name == None :
    an_option_parser.error( "Use --study-name option to define proper value\n" )
    pass
print_d( "a_study_name = '%s'\n" % a_study_name )

a_located_files = None
if len( an_args ) == 0 :
    a_located_files = [ raw_input() ]
else :
    a_located_files = an_args
    pass

if a_located_files == None :
    an_option_parser.error( "Define 'located files' to be removed - nothing to be done for now\n" )
    pass

print_d( "a_located_files = %r\n" % a_located_files )

common.extract_options( an_options )

AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY = amazon.extract_options( an_options )

a_number_threads = amazon.extract_threading_options( an_options, an_option_parser )

amazon.extract_timeout_options( an_options, an_option_parser )


print_i( "--------------------------- Looking for study object ----------------------------\n" )
a_root_object = TRootObject.get( AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY )
print_d( "a_root_object = %s\n" % a_root_object )

a_study_object = TStudyObject.get( a_root_object, a_study_name )
print_d( "a_study_object = %s\n" % a_study_object )


print_i( "-------------------------- Removing study files -----------------------------\n" )
for a_located_file in a_located_files :
    print_d( "a_located_file = %s\n" % a_located_file )
    
    a_file_object = TFileObject.get( a_study_object, a_located_file )
    print_d( "a_file_object = %s\n" % a_file_object, 1 )

    a_file_object.delete()
    
    pass


print_i( "-------------------------------------- OK ---------------------------------------\n" )




#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
from balloon.common import print_e, print_d, Timer


#--------------------------------------------------------------------------------------
def region_connect( the_image_location, the_aws_access_key_id, the_aws_secret_access_key ) :
    import boto.ec2
    a_regions = boto.ec2.regions()
    print_d( 'a_regions = %s\n' % [ str( a_region.name ) for a_region in a_regions ] )

    an_image_region = None
    for a_region in a_regions :
        if a_region.name == the_image_location :
            an_image_region = a_region
            pass
        pass
    if an_image_region == None :
        print_e( "There no region with such location - '%s'\n" % an_image_region )
        pass
    print_d( 'an_image_region = < %r >\n' % an_image_region )
    
    an_ec2_conn = an_image_region.connect( aws_access_key_id = the_aws_access_key_id, 
                                           aws_secret_access_key = the_aws_secret_access_key )
    print_d( 'an_ec2_conn = < %r >\n' % an_ec2_conn )

    return an_ec2_conn


#--------------------------------------------------------------------------------------



#!/usr/bin/env python

#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
"""
This script is responsible for the task packaging and sending it for execution in a cloud
"""

#--------------------------------------------------------------------------------------
import sys, os, os.path

# To avoid using previoulsy cached contents for the distributed package
an_engine = sys.argv[ 0 ]
an_engine_dir = os.path.abspath( os.path.dirname( sys.argv[ 0 ] ) )
a_manifest_file = os.path.join( an_engine_dir, 'MANIFEST' )
if os.path.isfile( a_manifest_file ) :
    os.remove( a_manifest_file )
    pass

an_engine = os.path.basename( an_engine )

# To generate list of automatically distributed scripts
a_scripts = []
for a_file in os.listdir( an_engine_dir ) :
    if a_file == an_engine :
        continue

    if os.path.isfile( a_file ) : 
        if os.access( a_file, os.X_OK ) :
            a_scripts.append( a_file )
            pass
        pass
    pass


#-------------------------------------------------------------------------------------
import ez_setup
ez_setup.use_setuptools( '0.6c9' )

from setuptools import setup, find_packages
import balloon

os.chdir( an_engine_dir ) # Run from the proper folder

setup( name = balloon.NAME,
       description = 'Set of cloud computing automation utilities',
       long_description = 
       """These utilities provide seemless mode for:
        - accessing to cloud;
        - deploying user specified data and functionality to be run on this data;
        - launching of the cloud specified task;
        - storing of output data into cloud;
        - fectching of these output results from cloud;
        - other miscellaneous functions""",
       author = 'Alexey Petrov',
       author_email = 'alexey.petrov.nnov@gmail.com', 
       license = 'Apache License, Version 2.0',
       url = 'http://sourceforge.net/p/cloud-foam',
       install_requires = [ 'boto == 1.9b', 'workerpool >= 0.9.2', 'paramiko >= 1.7.6' ],
       platforms = [ 'linux' ],
       version = balloon.VERSION,
       classifiers = [ 'Development Status :: 3 - Alpha',
                       'Environment :: Console',
                       'Intended Audience :: Science/Research',
                       'License :: OSI Approved :: Apache Software License',
                       'Operating System :: POSIX',
                       'Programming Language :: Python',
                       'Topic :: Scientific/Engineering',
                       'Topic :: Utilities' ],
       packages = find_packages(),
       scripts = a_scripts,
       zip_safe = True )


#--------------------------------------------------------------------------------------





#--------------------------------------------------------------------------------------
## Copyright 2010 Alexey Petrov
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##     http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.
##
## See http://sourceforge.net/apps/mediawiki/balloon-foam
##
## Author : Alexey Petrov
##


#--------------------------------------------------------------------------------------
import os, os.path, sys, hashlib
from subprocess import *


#--------------------------------------------------------------------------------------
ENABLE_DEBUG = False


#--------------------------------------------------------------------------------------
def add_usage_description() :
    return " --debug"


#--------------------------------------------------------------------------------------
def add_parser_options( the_option_parser, the_default_value = ENABLE_DEBUG ) :
    the_option_parser.add_option( "--debug",
                                  action = "store_true",
                                  dest = "enable_debug",
                                  help = "print debug information",
                                  default = the_default_value )
    pass


#--------------------------------------------------------------------------------------
def extract_options( the_options ) :
    global ENABLE_DEBUG

    ENABLE_DEBUG = the_options.enable_debug

    if the_options.enable_debug :
        import os.path
        a_filename = os.path.expanduser( "~/balloon.log" )

        import logging
        logging.basicConfig( filename = a_filename, level = logging.DEBUG )

        pass

    return ENABLE_DEBUG


#---------------------------------------------------------------------------
PRINTING_DEPTH = 0

def get_preffix( the_preffix, the_printing_depth ) :
    "Calaculate a preffix identation according to the current printing stack depth"
    a_preffix = ""
    for id in range( the_printing_depth ) :
        a_preffix += the_preffix
        pass

    return a_preffix


#---------------------------------------------------------------------------
def print_d( the_message, the_printing_depth = 0 ) :
    "Optional printing of debug messages"
    if ENABLE_DEBUG : 
        sys.stderr.write( get_preffix( "    ", the_printing_depth ) + the_message )
        pass
    
    pass


#---------------------------------------------------------------------------
def print_i( the_message, the_printing_depth = 0 ) :
    "Optional printing of debug messages"

    print_d( "\n" )

    print_d( the_message, the_printing_depth )

    pass


#---------------------------------------------------------------------------
def print_list( the_caption, the_list, the_numbering = True, the_printing_depth = 0 ) :
    "Optional printing of debug messages"
    if ENABLE_DEBUG : 
        sys.stderr.write( get_preffix( "    ", the_printing_depth ) + the_caption )
        if the_numbering == True :
            for an_id in range( len( the_list ) ) :
                an_item = the_list[ an_id ]
                sys.stderr.write( get_preffix( "    ", the_printing_depth + 1 ) + "%02d : [ %s ]\n" % ( an_id, an_item ) )
                pass
        else :
            for an_item in the_list :
                an_item = the_list[ an_id ]
                sys.stderr.write( get_preffix( "    ", the_printing_depth + 1 ) + "[ %s ]\n" % ( an_item ) )
                pass
            pass
        pass
    pass


#---------------------------------------------------------------------------
def print_e( the_message ) :
    "Printing of error message and quit"
    print_d( "\n---------------------------------------------------------------------------\n" )

    sys.stderr.write( "Error : " + the_message )
    
    print_d( "\n---------------------------------------------------------------------------\n" )
    print_d( 'KO\n' )

    os._exit( os.EX_USAGE )
    pass


#---------------------------------------------------------------------------
def sh_command( the_command, the_printing_depth = 0 ) :
    "Execution of shell command"
    print_d( "(%s)\n" % the_command, the_printing_depth )
    
    a_pipe = Popen( the_command, stdout = PIPE, stderr = PIPE, shell = True )

    a_return_code = os.waitpid( a_pipe.pid, 0 )[ 1 ]

    a_stderr_lines = a_pipe.stderr.readlines()
    for a_line in a_stderr_lines : print_d( "\t%s" % a_line )

    a_stdout_lines = a_pipe.stdout.readlines()
    for a_line in a_stdout_lines : print_d( "\t%s" % a_line )

    if a_return_code != 0 :
        os._exit( os.EX_USAGE )
        pass
    
    return a_stdout_lines


#---------------------------------------------------------------------------
def generate_queue_name( the_container_name, the_queue_suffix ) :
    "Generating composite queue name"
    
    return '%s___%s' % ( the_container_name, the_queue_suffix )


#---------------------------------------------------------------------------
def generate_initial_queue_name( the_container_name ) :
    "Defining the initial queue name"
    a_queue_suffix = 'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx'
    
    return generate_queue_name( the_container_name, a_queue_suffix )


#---------------------------------------------------------------------------
def generate_message_body( the_file_name, the_next_queue_suffix ) :
    "Generating specially formatted message"
    
    return '%s:%s' % ( the_file_name, the_next_queue_suffix )


#---------------------------------------------------------------------------
def generate_final_message_body( the_container_name ) :
    "Defining the final message"
    
    return generate_message_body( the_container_name, '*' )


#--------------------------------------------------------------------------------------
def push_message( the_sqs_queue, the_string ) :
    "Publishing message through Amazon SQS"
    from boto.sqs.message import Message

    a_message = Message()
    a_message.set_body( the_string )

    return the_sqs_queue.write( a_message )


#--------------------------------------------------------------------------------------
def get_message( the_sqs_conn, the_queue_name ) :
    "Receiving and parsing the incoming Amazon SQS message"

    # Get the 'queue' first
    a_sqs_queue = None
    while True :
        a_sqs_queue = the_sqs_conn.get_queue( the_queue_name )
        if a_sqs_queue != None :
            break
        pass

    # Now, get message
    a_message_body = ''
    while True :
        a_message = a_sqs_queue.read()
        if a_message != None :
            a_message_body = a_message.get_body()
            break
        pass

    a_sqs_queue.delete()

    a_data_name, a_next_queue_suffix = a_message_body.split( ':' )

    an_is_final = False
    if a_next_queue_suffix == '*':
        an_is_final = True
        pass

    return an_is_final, a_data_name, a_next_queue_suffix


#--------------------------------------------------------------------------------------
class Timer :
    def __init__( self ) :
        import time
        self.initial = time.time()
        pass

    def __str__( self ) :
        import time
        return str( "%f" % ( time.time() - self.initial ))

    pass


#------------------------------------------------------------------------------------------
import workerpool
from workerpool.pools import default_worker_factory

class WorkerPool( workerpool.WorkerPool ) :
    "Run all the registered tasks in parallel"
    def __init__( self, size = 1, maxjobs = 0, worker_factory = default_worker_factory ) :
        workerpool.WorkerPool.__init__( self, size, maxjobs, worker_factory )

        from workerpool.pools import Queue
        self.results = Queue()

        pass

    def charge( self, the_function, *the_args ):
        "Perform a map operation distributed among the workers. Will block until done."

        from workerpool import SimpleJob
        self.put( SimpleJob( self.results, the_function, *the_args ) )

        pass

    def is_all_right( self ) :
        self.join()

        a_result = True
        for i in range( self.results.qsize() ):
            a_result &= self.results.get()
            self.results.task_done()
            pass

        return a_result

    pass


#--------------------------------------------------------------------------------------
def compute_md5( the_file_pointer ) :
    from boto.s3.key import Key

    return Key().compute_md5( the_file_pointer )


#--------------------------------------------------------------------------------------
def arg_list_separator():
     return ","


#--------------------------------------------------------------------------------------
